# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/tools/settingseditor example from Qt v5.x"""

import sys

from PySide6.QtCore import (QByteArray, QDate, QDateTime, QDir, QEvent, QPoint,
                            QRect, QRegularExpression, QSettings, QSize, QTime,
                            QTimer, Qt, Slot)
from PySide6.QtGui import (QAction, QColor, QIcon, QIntValidator,
                           QDoubleValidator, QRegularExpressionValidator,
                           QValidator)
from PySide6.QtWidgets import (QAbstractItemView, QApplication,
                               QCheckBox, QComboBox, QFileDialog, QDialog,
                               QDialogButtonBox, QGridLayout,
                               QGroupBox, QHeaderView, QInputDialog,
                               QItemDelegate, QLabel, QLineEdit,
                               QMainWindow, QMessageBox, QStyle, QSpinBox,
                               QStyleOptionViewItem, QTableWidget,
                               QTableWidgetItem, QTreeWidget, QTreeWidgetItem,
                               QVBoxLayout)


class TypeChecker:
    def __init__(self, parent=None):
        self.bool_exp = QRegularExpression('^(true)|(false)$')
        assert self.bool_exp.isValid()
        self.bool_exp.setPatternOptions(QRegularExpression.PatternOption.CaseInsensitiveOption)

        self.byteArray_exp = QRegularExpression(r'^[\x00-\xff]*$')
        assert self.byteArray_exp.isValid()

        self.char_exp = QRegularExpression('^.$')
        assert self.char_exp.isValid()

        pattern = r'^[+-]?\d+$'
        self.int_exp = QRegularExpression(pattern)
        assert self.int_exp.isValid()

        pattern = r'^\(([0-9]*),([0-9]*),([0-9]*),([0-9]*)\)$'
        self.color_exp = QRegularExpression(pattern)
        assert self.color_exp.isValid()

        pattern = r'^\((-?[0-9]*),(-?[0-9]*)\)$'
        self.point_exp = QRegularExpression(pattern)
        assert self.point_exp.isValid()

        pattern = r'^\((-?[0-9]*),(-?[0-9]*),(-?[0-9]*),(-?[0-9]*)\)$'
        self.rect_exp = QRegularExpression(pattern)
        assert self.rect_exp.isValid()

        self.size_exp = QRegularExpression(self.point_exp)

        date_pattern = '([0-9]{,4})-([0-9]{,2})-([0-9]{,2})'
        self.date_exp = QRegularExpression(f'^{date_pattern}$')
        assert self.date_exp.isValid()

        time_pattern = '([0-9]{,2}):([0-9]{,2}):([0-9]{,2})'
        self.time_exp = QRegularExpression(f'^{time_pattern}$')
        assert self.time_exp.isValid()

        pattern = f'^{date_pattern}T{time_pattern}$'
        self.dateTime_exp = QRegularExpression(pattern)
        assert self.dateTime_exp.isValid()

    def type_from_text(self, text):
        if self.bool_exp.match(text).hasMatch():
            return bool
        if self.int_exp.match(text).hasMatch():
            return int
        return None

    def create_validator(self, value, parent):
        if isinstance(value, bool):
            return QRegularExpressionValidator(self.bool_exp, parent)
        if isinstance(value, float):
            return QDoubleValidator(parent)
        if isinstance(value, int):
            return QIntValidator(parent)
        if isinstance(value, QByteArray):
            return QRegularExpressionValidator(self.byteArray_exp, parent)
        if isinstance(value, QColor):
            return QRegularExpressionValidator(self.color_exp, parent)
        if isinstance(value, QDate):
            return QRegularExpressionValidator(self.date_exp, parent)
        if isinstance(value, QDateTime):
            return QRegularExpressionValidator(self.dateTime_exp, parent)
        if isinstance(value, QTime):
            return QRegularExpressionValidator(self.time_exp, parent)
        if isinstance(value, QPoint):
            return QRegularExpressionValidator(self.point_exp, parent)
        if isinstance(value, QRect):
            return QRegularExpressionValidator(self.rect_exp, parent)
        if isinstance(value, QSize):
            return QRegularExpressionValidator(self.size_exp, parent)
        return None

    def from_string(self, text, original_value):
        if isinstance(original_value, QColor):
            match = self.color_exp.match(text)
            return QColor(min(int(match.captured(1)), 255),
                          min(int(match.captured(2)), 255),
                          min(int(match.captured(3)), 255),
                          min(int(match.captured(4)), 255))
        if isinstance(original_value, QDate):
            value = QDate.fromString(text, Qt.DateFormat.ISODate)
            return value if value.isValid() else None
        if isinstance(original_value, QDateTime):
            value = QDateTime.fromString(text, Qt.DateFormat.ISODate)
            return value if value.isValid() else None
        if isinstance(original_value, QTime):
            value = QTime.fromString(text, Qt.DateFormat.ISODate)
            return value if value.isValid() else None
        if isinstance(original_value, QPoint):
            match = self.point_exp.match(text)
            return QPoint(int(match.captured(1)),
                          int(match.captured(2)))
        if isinstance(original_value, QRect):
            match = self.rect_exp.match(text)
            return QRect(int(match.captured(1)),
                         int(match.captured(2)),
                         int(match.captured(3)),
                         int(match.captured(4)))
        if isinstance(original_value, QSize):
            match = self.size_exp.match(text)
            return QSize(int(match.captured(1)),
                         int(match.captured(2)))
        if isinstance(original_value, list):
            return text.split(',')
        return type(original_value)(text)


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.settings_tree = SettingsTree()
        self.setCentralWidget(self.settings_tree)

        self.location_dialog = None

        self.create_actions()
        self.create_menus()

        self.auto_refresh_action.setChecked(True)
        self.fallbacks_action.setChecked(True)

        self.setWindowTitle("Settings Editor")
        self.resize(500, 600)

    @Slot()
    def open_settings(self):
        if self.location_dialog is None:
            self.location_dialog = LocationDialog(self)

        if self.location_dialog.exec():
            settings = QSettings(self.location_dialog.format(),
                                 self.location_dialog.scope(),
                                 self.location_dialog.organization(),
                                 self.location_dialog.application())
            self.set_settings_object(settings)
            self.fallbacks_action.setEnabled(True)

    @Slot()
    def open_inifile(self):
        file_name, _ = QFileDialog.getOpenFileName(self, "Open INI File",
                                                   '', "INI Files (*.ini *.conf)")

        if file_name:
            self.load_ini_file(file_name)

    def load_ini_file(self, file_name):
        settings = QSettings(file_name, QSettings.Format.IniFormat)
        if settings.status() != QSettings.Status.NoError:
            return
        self.set_settings_object(settings)
        self.fallbacks_action.setEnabled(False)

    @Slot()
    def open_property_list(self):
        file_name, _ = QFileDialog.getOpenFileName(self,
                                                   "Open Property List", '',
                                                   "Property List Files (*.plist)")

        if file_name:
            settings = QSettings(file_name, QSettings.Format.NativeFormat)
            self.set_settings_object(settings)
            self.fallbacks_action.setEnabled(False)

    @Slot()
    def open_registry_path(self):
        path, ok = QInputDialog.getText(self, "Open Registry Path",
                                        "Enter the path in the Windows registry:",
                                        QLineEdit.Normal, 'HKEY_CURRENT_USER\\')

        if ok and path != '':
            settings = QSettings(path, QSettings.Format.NativeFormat)
            self.set_settings_object(settings)
            self.fallbacks_action.setEnabled(False)

    @Slot()
    def about(self):
        QMessageBox.about(self, "About Settings Editor",
                          "The <b>Settings Editor</b> example shows how to access "
                          "application settings using Qt.")

    def create_actions(self):
        self.open_settings_action = QAction("&Open Application Settings...",
                                            self, shortcut="Ctrl+O", triggered=self.open_settings)

        self.open_ini_file_action = QAction("Open I&NI File...", self,
                                            shortcut="Ctrl+N", triggered=self.open_inifile)

        self.open_property_list_action = QAction("Open macOS &Property List...", self,
                                                 shortcut="Ctrl+P",
                                                 triggered=self.open_property_list)
        if sys.platform != 'darwin':
            self.open_property_list_action.setEnabled(False)

        self.open_registry_path_action = QAction(
            "Open Windows &Registry Path...", self, shortcut="Ctrl+G",
            triggered=self.open_registry_path)
        if sys.platform != 'win32':
            self.open_registry_path_action.setEnabled(False)

        self.refresh_action = QAction("&Refresh", self, shortcut="Ctrl+R",
                                      enabled=False, triggered=self.settings_tree.refresh)

        self.exit_action = QAction("E&xit", self, shortcut="Ctrl+Q", triggered=self.close)

        self.auto_refresh_action = QAction("&Auto-Refresh", self,
                                           shortcut="Ctrl+A", checkable=True, enabled=False)
        self.auto_refresh_action.triggered[bool].connect(self.settings_tree.set_auto_refresh)
        self.auto_refresh_action.triggered[bool].connect(self.refresh_action.setDisabled)

        self.fallbacks_action = QAction("&Fallbacks", self,
                                        shortcut="Ctrl+F", checkable=True, enabled=False)
        self.fallbacks_action.triggered[bool].connect(self.settings_tree.set_fallbacks_enabled)

        self.about_action = QAction("&About", self, triggered=self.about)

        self.about_Qt_action = QAction("About &Qt", self,
                                       triggered=qApp.aboutQt)  # noqa: F821

    def create_menus(self):
        self.file_menu = self.menuBar().addMenu("&File")
        self.file_menu.addAction(self.open_settings_action)
        self.file_menu.addAction(self.open_ini_file_action)
        self.file_menu.addAction(self.open_property_list_action)
        self.file_menu.addAction(self.open_registry_path_action)
        self.file_menu.addSeparator()
        self.file_menu.addAction(self.refresh_action)
        self.file_menu.addSeparator()
        self.file_menu.addAction(self.exit_action)

        self.options_menu = self.menuBar().addMenu("&Options")
        self.options_menu.addAction(self.auto_refresh_action)
        self.options_menu.addAction(self.fallbacks_action)

        self.menuBar().addSeparator()

        self.help_menu = self.menuBar().addMenu("&Help")
        self.help_menu.addAction(self.about_action)
        self.help_menu.addAction(self.about_Qt_action)

    def set_settings_object(self, settings):
        settings.setFallbacksEnabled(self.fallbacks_action.isChecked())
        self.settings_tree.set_settings_object(settings)

        self.refresh_action.setEnabled(True)
        self.auto_refresh_action.setEnabled(True)

        nice_name = QDir.fromNativeSeparators(settings.fileName())
        nice_name = nice_name.split('/')[-1]

        if not settings.isWritable():
            nice_name += " (read only)"

        self.setWindowTitle(f"{nice_name} - Settings Editor")


class LocationDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.format_combo = QComboBox()
        self.format_combo.addItem("Native")
        self.format_combo.addItem("INI")

        self.scope_cCombo = QComboBox()
        self.scope_cCombo.addItem("User")
        self.scope_cCombo.addItem("System")

        self.organization_combo = QComboBox()
        self.organization_combo.addItem("Trolltech")
        self.organization_combo.setEditable(True)

        self.application_combo = QComboBox()
        self.application_combo.addItem("Any")
        self.application_combo.addItem("Application Example")
        self.application_combo.addItem("Assistant")
        self.application_combo.addItem("Designer")
        self.application_combo.addItem("Linguist")
        self.application_combo.setEditable(True)
        self.application_combo.setCurrentIndex(3)

        format_label = QLabel("&Format:")
        format_label.setBuddy(self.format_combo)

        scope_label = QLabel("&Scope:")
        scope_label.setBuddy(self.scope_cCombo)

        organization_label = QLabel("&Organization:")
        organization_label.setBuddy(self.organization_combo)

        application_label = QLabel("&Application:")
        application_label.setBuddy(self.application_combo)

        self.locations_groupbox = QGroupBox("Setting Locations")

        self.locations_table = QTableWidget()
        self.locations_table.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
        self.locations_table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.locations_table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.locations_table.setColumnCount(2)
        self.locations_table.setHorizontalHeaderLabels(("Location", "Access"))
        header = self.locations_table.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
        header.resizeSection(1, 180)

        self.button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok
                                           | QDialogButtonBox.StandardButton.Cancel)

        self.format_combo.activated.connect(self.update_locations)
        self.scope_cCombo.activated.connect(self.update_locations)
        self.organization_combo.lineEdit().editingFinished.connect(self.update_locations)
        self.application_combo.lineEdit().editingFinished.connect(self.update_locations)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        locations_layout = QVBoxLayout(self.locations_groupbox)
        locations_layout.addWidget(self.locations_table)

        main_layout = QGridLayout(self)
        main_layout.addWidget(format_label, 0, 0)
        main_layout.addWidget(self.format_combo, 0, 1)
        main_layout.addWidget(scope_label, 1, 0)
        main_layout.addWidget(self.scope_cCombo, 1, 1)
        main_layout.addWidget(organization_label, 2, 0)
        main_layout.addWidget(self.organization_combo, 2, 1)
        main_layout.addWidget(application_label, 3, 0)
        main_layout.addWidget(self.application_combo, 3, 1)
        main_layout.addWidget(self.locations_groupbox, 4, 0, 1, 2)
        main_layout.addWidget(self.button_box, 5, 0, 1, 2)

        self.update_locations()

        self.setWindowTitle("Open Application Settings")
        self.resize(650, 400)

    def format(self):
        if self.format_combo.currentIndex() == 0:
            return QSettings.Format.NativeFormat
        else:
            return QSettings.Format.IniFormat

    def scope(self):
        if self.scope_cCombo.currentIndex() == 0:
            return QSettings.Scope.UserScope
        else:
            return QSettings.Scope.SystemScope

    def organization(self):
        return self.organization_combo.currentText()

    def application(self):
        if self.application_combo.currentText() == "Any":
            return ''

        return self.application_combo.currentText()

    def update_locations(self):
        self.locations_table.setUpdatesEnabled(False)
        self.locations_table.setRowCount(0)

        for i in range(2):
            if i == 0:
                if self.scope() == QSettings.Scope.SystemScope:
                    continue

                actual_scope = QSettings.Scope.UserScope
            else:
                actual_scope = QSettings.Scope.SystemScope

            for j in range(2):
                if j == 0:
                    if not self.application():
                        continue

                    actual_application = self.application()
                else:
                    actual_application = ''

                settings = QSettings(self.format(), actual_scope,
                                     self.organization(), actual_application)

                row = self.locations_table.rowCount()
                self.locations_table.setRowCount(row + 1)

                item0 = QTableWidgetItem()
                item0.setText(settings.fileName())

                item1 = QTableWidgetItem()
                disable = not (settings.childKeys() or settings.childGroups())

                if row == 0:
                    if settings.isWritable():
                        item1.setText("Read-write")
                        disable = False
                    else:
                        item1.setText("Read-only")
                    self.button_box.button(QDialogButtonBox.StandardButton.Ok).setDisabled(disable)
                else:
                    item1.setText("Read-only fallback")

                if disable:
                    item0.setFlags(item0.flags() & ~Qt.ItemFlag.ItemIsEnabled)
                    item1.setFlags(item1.flags() & ~Qt.ItemFlag.ItemIsEnabled)

                self.locations_table.setItem(row, 0, item0)
                self.locations_table.setItem(row, 1, item1)

        self.locations_table.setUpdatesEnabled(True)


class SettingsTree(QTreeWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self._type_checker = TypeChecker()
        self.setItemDelegate(VariantDelegate(self._type_checker, self))

        self.setHeaderLabels(("Setting", "Type", "Value"))
        self.header().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
        self.header().setSectionResizeMode(2, QHeaderView.ResizeMode.Stretch)

        self.settings = None
        self.refresh_timer = QTimer()
        self.refresh_timer.setInterval(2000)
        self.auto_refresh = False

        self.group_icon = QIcon()
        style = self.style()
        self.group_icon.addPixmap(style.standardPixmap(QStyle.StandardPixmap.SP_DirClosedIcon),
                                  QIcon.Mode.Normal, QIcon.State.Off)
        self.group_icon.addPixmap(style.standardPixmap(QStyle.StandardPixmap.SP_DirOpenIcon),
                                  QIcon.Mode.Normal, QIcon.State.On)
        self.key_icon = QIcon()
        self.key_icon.addPixmap(style.standardPixmap(QStyle.StandardPixmap.SP_FileIcon))

        self.refresh_timer.timeout.connect(self.maybe_refresh)

    def set_settings_object(self, settings):
        self.settings = settings
        self.clear()

        if self.settings is not None:
            self.settings.setParent(self)
            self.refresh()
            if self.auto_refresh:
                self.refresh_timer.start()
        else:
            self.refresh_timer.stop()

    def sizeHint(self):
        return QSize(800, 600)

    @Slot(bool)
    def set_auto_refresh(self, autoRefresh):
        self.auto_refresh = autoRefresh

        if self.settings is not None:
            if self.auto_refresh:
                self.maybe_refresh()
                self.refresh_timer.start()
            else:
                self.refresh_timer.stop()

    @Slot(bool)
    def set_fallbacks_enabled(self, enabled):
        if self.settings is not None:
            self.settings.setFallbacksEnabled(enabled)
            self.refresh()

    @Slot()
    def maybe_refresh(self):
        if self.state() != QAbstractItemView.EditingState:
            self.refresh()

    @Slot()
    def refresh(self):
        if self.settings is None:
            return

        # The signal might not be connected.
        try:
            self.itemChanged.disconnect(self.update_setting)
        except Exception:
            pass

        self.settings.sync()
        self.update_child_items(None)

        self.itemChanged.connect(self.update_setting)

    def event(self, event):
        if event.type() == QEvent.Type.WindowActivate:
            if self.isActiveWindow() and self.auto_refresh:
                self.maybe_refresh()

        return super(SettingsTree, self).event(event)

    def update_setting(self, item):
        key = item.text(0)
        ancestor = item.parent()

        while ancestor:
            key = ancestor.text(0) + '/' + key
            ancestor = ancestor.parent()

        self.settings.setValue(key, item.data(2, Qt.ItemDataRole.UserRole))

        if self.auto_refresh:
            self.refresh()

    def update_child_items(self, parent):
        divider_index = 0

        for group in self.settings.childGroups():
            child_index = self.find_child(parent, group, divider_index)
            if child_index != -1:
                child = self.child_at(parent, child_index)
                child.setText(1, '')
                child.setText(2, '')
                child.setData(2, Qt.ItemDataRole.UserRole, None)
                self.move_item_forward(parent, child_index, divider_index)
            else:
                child = self.create_item(group, parent, divider_index)

            child.setIcon(0, self.group_icon)
            divider_index += 1

            self.settings.beginGroup(group)
            self.update_child_items(child)
            self.settings.endGroup()

        for key in self.settings.childKeys():
            child_index = self.find_child(parent, key, 0)
            if child_index == -1 or child_index >= divider_index:
                if child_index != -1:
                    child = self.child_at(parent, child_index)
                    for i in range(child.childCount()):
                        self.delete_item(child, i)
                    self.move_item_forward(parent, child_index, divider_index)
                else:
                    child = self.create_item(key, parent, divider_index)
                child.setIcon(0, self.key_icon)
                divider_index += 1
            else:
                child = self.child_at(parent, child_index)

            value = self.settings.value(key)
            if value is None:
                child.setText(1, 'Invalid')
            else:
                # Try to convert to type unless a QByteArray is received
                if isinstance(value, str):
                    if value_type := self._type_checker.type_from_text(value):
                        value = self.settings.value(key, type=value_type)
                child.setText(1, value.__class__.__name__)
            child.setText(2, VariantDelegate.display_text(value))
            child.setData(2, Qt.ItemDataRole.UserRole, value)

        while divider_index < self.child_count(parent):
            self.delete_item(parent, divider_index)

    def create_item(self, text, parent, index):
        after = None

        if index != 0:
            after = self.child_at(parent, index - 1)

        if parent is not None:
            item = QTreeWidgetItem(parent, after)
        else:
            item = QTreeWidgetItem(self, after)

        item.setText(0, text)
        item.setFlags(item.flags() | Qt.ItemFlag.ItemIsEditable)
        return item

    def delete_item(self, parent, index):
        if parent is not None:
            item = parent.takeChild(index)
        else:
            item = self.takeTopLevelItem(index)
        del item

    def child_at(self, parent, index):
        if parent is not None:
            return parent.child(index)
        else:
            return self.topLevelItem(index)

    def child_count(self, parent):
        if parent is not None:
            return parent.childCount()
        else:
            return self.topLevelItemCount()

    def find_child(self, parent, text, startIndex):
        for i in range(self.child_count(parent)):
            if self.child_at(parent, i).text(0) == text:
                return i
        return -1

    def move_item_forward(self, parent, oldIndex, newIndex):
        for int in range(oldIndex - newIndex):
            self.delete_item(parent, newIndex)


class VariantDelegate(QItemDelegate):
    def __init__(self, type_checker, parent=None):
        super().__init__(parent)
        self._type_checker = type_checker

    def paint(self, painter, option, index):
        if index.column() == 2:
            value = index.model().data(index, Qt.ItemDataRole.UserRole)
            if not self.is_supported_type(value):
                my_option = QStyleOptionViewItem(option)
                my_option.state &= ~QStyle.State_Enabled
                super(VariantDelegate, self).paint(painter, my_option, index)
                return

        super(VariantDelegate, self).paint(painter, option, index)

    def createEditor(self, parent, option, index):
        if index.column() != 2:
            return None

        original_value = index.model().data(index, Qt.ItemDataRole.UserRole)
        if not self.is_supported_type(original_value):
            return None

        editor = None
        if isinstance(original_value, bool):
            editor = QCheckBox(parent)
        if isinstance(original_value, int):
            editor = QSpinBox(parent)
            editor.setRange(-32767, 32767)
        else:
            editor = QLineEdit(parent)
            editor.setFrame(False)
            if validator := self._type_checker.create_validator(original_value, editor):
                editor.setValidator(validator)
        return editor

    def setEditorData(self, editor, index):
        if not editor:
            return
        value = index.model().data(index, Qt.ItemDataRole.UserRole)
        if isinstance(editor, QCheckBox):
            editor.setCheckState(Qt.Checked if value else Qt.Unchecked)
        elif isinstance(editor, QSpinBox):
            editor.setValue(value)
        else:
            editor.setText(self.display_text(value))

    def value_from_lineedit(self, lineedit, model, index):
        if not lineedit.isModified():
            return None
        text = lineedit.text()
        validator = lineedit.validator()
        if validator is not None:
            state, text, _ = validator.validate(text, 0)
            if state != QValidator.Acceptable:
                return None
        original_value = index.model().data(index, Qt.ItemDataRole.UserRole)
        return self._type_checker.from_string(text, original_value)

    def setModelData(self, editor, model, index):
        value = None
        if isinstance(editor, QCheckBox):
            value = editor.checkState() == Qt.Checked
        elif isinstance(editor, QSpinBox):
            value = editor.value()
        else:
            value = self.value_from_lineedit(editor, model, index)
        if value is not None:
            model.setData(index, value, Qt.ItemDataRole.UserRole)
            model.setData(index, self.display_text(value), Qt.ItemDataRole.DisplayRole)

    @staticmethod
    def is_supported_type(value):
        return isinstance(value, (bool, float, int, QByteArray, str, QColor,
                                  QDate, QDateTime, QTime, QPoint, QRect,
                                  QSize, list))

    @staticmethod
    def display_text(value):
        if isinstance(value, str):
            return value
        if isinstance(value, bool):
            return '✓' if value else '☐'
        if isinstance(value, (int, float, QByteArray)):
            return str(value)
        if isinstance(value, QColor):
            (r, g, b, a) = (value.red(), value.green(), value.blue(), value.alpha())
            return f'({r},{g},{b},{a})'
        if isinstance(value, (QDate, QDateTime, QTime)):
            return value.toString(Qt.ISODate)
        if isinstance(value, QPoint):
            x = value.x()
            y = value.y()
            return f'({x},{y})'
        if isinstance(value, QRect):
            x = value.x()
            y = value.y()
            w = value.width()
            h = value.height()
            return f'({x},{y},{w},{h})'
        if isinstance(value, QSize):
            w = value.width()
            h = value.height()
            return f'({w},{h})'
        if isinstance(value, list):
            return ','.join(map(repr, value))
        if value is None:
            return '<Invalid>'

        return f'<{value}>'


if __name__ == '__main__':
    app = QApplication(sys.argv)
    main_win = MainWindow()
    if len(sys.argv) > 1:
        main_win.load_ini_file(sys.argv[1])
    main_win.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.timedqguiapplication import TimedQGuiApplication
from PySide6.QtGui import QIcon


class QIconCtorWithNoneTest(TimedQGuiApplication):
    '''Test made by seblin, see Bug #944: http://bugs.pyside.org/show_bug.cgi?id=944'''

    def testQIconCtorWithNone(self):
        icon = QIcon(None)
        pixmap = icon.pixmap(48, 48)  # noqa: F841
        self.app.exec()


PIX_PATH = os.fspath(Path(__file__).resolve().parents[2]
                     / "doc/tutorials/basictutorial/icons.png")


class QIconAddPixmapTest(TimedQGuiApplication):
    '''PYSIDE-1669: check that addPixmap works'''

    def testQIconSetPixmap(self):
        icon = QIcon()
        icon.addPixmap(PIX_PATH)
        sizes = icon.availableSizes()
        self.assertTrue(sizes)

    def testQIconSetPixmapPathlike(self):
        icon = QIcon()
        pix_path = Path(PIX_PATH)
        icon.addPixmap(pix_path)
        sizes = icon.availableSizes()
        self.assertTrue(sizes)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test bug 585: http://bugs.openbossa.org/show_bug.cgi?id=585'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QTreeWidget, QTreeWidgetItem


class Bug585(unittest.TestCase):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testCase(self):
        app = QApplication([])  # noqa: F841
        self._tree = QTreeWidget()
        self._tree.setColumnCount(2)
        i1 = QTreeWidgetItem(self._tree, ['1', ])
        i2 = QTreeWidgetItem(self._tree, ['2', ])  # noqa: F841
        refCount = sys.getrefcount(i1)

        # this function return None
        # because the topLevelItem does not has a parent item
        # but still have a TreeWidget as a parent
        self._tree.topLevelItem(0).parent()

        self.assertEqual(refCount, sys.getrefcount(i1))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2011 Arun Srinivasan <rulfzid@gmail.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import (Qt, QAbstractTableModel, QModelIndex)


class TableModel(QAbstractTableModel):

    def __init__(self, addresses=None, parent=None):
        super().__init__(parent)

        if addresses is None:
            self.addresses = []
        else:
            self.addresses = addresses

    def rowCount(self, index=QModelIndex()):
        """ Returns the number of rows the model holds. """
        return len(self.addresses)

    def columnCount(self, index=QModelIndex()):
        """ Returns the number of columns the model holds. """
        return 2

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        """ Depending on the index and role given, return data. If not
            returning data, return None (PySide equivalent of QT's
            "invalid QVariant").
        """
        if not index.isValid():
            return None

        if not 0 <= index.row() < len(self.addresses):
            return None

        if role == Qt.ItemDataRole.DisplayRole:
            name = self.addresses[index.row()]["name"]
            address = self.addresses[index.row()]["address"]

            if index.column() == 0:
                return name
            elif index.column() == 1:
                return address

        return None

    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
        """ Set the headers to be displayed. """
        if role != Qt.ItemDataRole.DisplayRole:
            return None

        if orientation == Qt.Orientation.Horizontal:
            if section == 0:
                return "Name"
            elif section == 1:
                return "Address"

        return None

    def insertRows(self, position, rows=1, index=QModelIndex()):
        """ Insert a row into the model. """
        self.beginInsertRows(QModelIndex(), position, position + rows - 1)

        for row in range(rows):
            self.addresses.insert(position + row, {"name": "", "address": ""})

        self.endInsertRows()
        return True

    def removeRows(self, position, rows=1, index=QModelIndex()):
        """ Remove a row from the model. """
        self.beginRemoveRows(QModelIndex(), position, position + rows - 1)

        del self.addresses[position:position + rows]

        self.endRemoveRows()
        return True

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        """ Adjust the data (set it to <value>) depending on the given
            index and role.
        """
        if role != Qt.ItemDataRole.EditRole:
            return False

        if index.isValid() and 0 <= index.row() < len(self.addresses):
            address = self.addresses[index.row()]
            if index.column() == 0:
                address["name"] = value
            elif index.column() == 1:
                address["address"] = value
            else:
                return False

            self.dataChanged.emit(index, index, 0)
            return True

        return False

    def flags(self, index):
        """ Set the item flags at the given index. Seems like we're
            implementing this function just to see how it's done, as we
            manually adjust each tableView to have NoEditTriggers.
        """
        if not index.isValid():
            return Qt.ItemFlag.ItemIsEnabled
        return Qt.ItemFlags(QAbstractTableModel.flags(self, index)
                            | Qt.ItemFlag.ItemIsEditable)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for a class that holds an arbitraty pointer and is modified to hold an PyObject.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import PointerHolder


class TestPointerHolder(unittest.TestCase):
    '''Test cases for a class that holds an arbitraty pointer and
       is modified to hold an PyObject.'''

    def testStoringAndRetrievingPointer(self):
        ph = PointerHolder('Hello')
        self.assertEqual(ph.pointer(), 'Hello')
        a = (1, 2, 3)
        ph = PointerHolder(a)
        self.assertEqual(ph.pointer(), a)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testReferenceCounting(self):
        '''Test reference counting when retrieving data with PointerHolder.pointer().'''
        a = (1, 2, 3)
        refcnt = sys.getrefcount(a)
        ph = PointerHolder(a)
        ptr = ph.pointer()  # noqa: F841
        self.assertEqual(sys.getrefcount(a), refcnt + 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations


import json
import sys
from dataclasses import Field, dataclass, field


_pyside_package_path = None
_module_json_file_path = None
_multimedia_libs = ["avcodec", "avformat", "avutil", "swresample", "swscale"]


def set_pyside_package_path(p):
    global _pyside_package_path, _module_json_file_path
    _pyside_package_path = p
    qt_path = p
    if sys.platform != "win32":
        qt_path /= "Qt"
    _module_json_file_path = qt_path / "modules"


def get_module_json_data(module):
    """Read the JSON module data."""
    json_path = _module_json_file_path / f"{module}.json"
    json_data = None
    if not json_path.is_file():  # Wayland is Linux only
        print(f"Skipping {json_path}", file=sys.stderr)
        return None
    with json_path.open(encoding="utf-8") as json_file:
        json_data = json.load(json_file)
    return json_data


def get_module_plugins(json_data):
    """Return the plugins from the JSON module data."""
    if json_data:
        plugins = json_data.get("plugin_types")
        if plugins:
            return plugins
    return []


# This dataclass is in charge of holding the file information
# that each Qt module needs to have to be packaged in a wheel
@dataclass
class ModuleData:
    name: str
    ext: str = ""
    # Libraries not related to Qt modules
    lib: list[str] = field(default_factory=list)
    # Libraries related to Qt modules
    qtlib: list[str] = field(default_factory=list)
    # Files from the Qt/qml directory
    qml: list[str] = field(default_factory=list)
    pyi: list[str] = field(default_factory=list)
    translations: list[str] = field(default_factory=list)
    typesystems: list[str] = field(default_factory=list)
    include: list[str] = field(default_factory=list)
    glue: list[str] = field(default_factory=list)
    doc_glue: list[str] = field(default_factory=list)
    metatypes: list[str] = field(default_factory=list)
    plugins: list[str] = field(default_factory=list)

    # For special cases when a file/directory doesn't fall into
    # the previous categories.
    extra_dirs: list[str] = field(default_factory=list)
    extra_files: list[str] = field(default_factory=list)

    # Once the object is created, this method will be executed
    # and automatically will initialize some of the files that are
    # common for each module.
    # Note: The goal of this list is to be used for a MANIFEST.in
    #       meaning that in case a file gets added and it doesn't
    #       exist, the wheel creation process will only throw a
    #       warning, but it will not interrupt the packaging process.
    def __post_init__(self) -> None:
        if not self.ext:
            self.ext = self.get_extension_from_platform(sys.platform)
        _lo = self.name.lower()

        self.lib.append(f"Qt{self.name}")
        self.qtlib.append(f"libQt6{self.name}")
        if not len(self.qml):
            self.qml.append(f"Qt{self.name}")
        self.pyi.append(f"Qt{self.name}.pyi")
        self.typesystems.append(f"typesystem_{_lo}.xml")
        self.include.append(f"Qt{self.name}/*.h")
        self.glue.append(f"qt{_lo}.cpp")
        self.doc_glue.append(f"qt{_lo}.rst")
        if not len(self.metatypes):
            self.metatypes.append(f"qt6{_lo}_metatypes.json")

    # The PySide6 directory that gets packaged by the build_scripts
    # 'prepare_packages()' has a certain structure that depends on
    # the platform. Because that directory is the base for the wheel
    # packaging to work, we use the relative paths that are included
    # on each file.
    # Note: The MANIFEST.in file doesn't need to have '\' or other
    #       separator, and respect the '/' even on Windows.
    def adjusts_paths_and_extensions(self) -> None:
        if sys.platform == "win32":
            self.lib = [f"{i}.*{self.ext}".replace("lib", "") for i in self.lib]
            self.qtlib = [f"{i}.*dll".replace("lib", "") for i in self.qtlib]
            self.qml = [f"qml/{i}" for i in self.qml]
            self.translations = [f"translations/{i}" for i in self.translations]
            self.metatypes = [f"metatypes/{i}" for i in self.metatypes]
            self.plugins = [f"plugins/{i}" for i in self.plugins]
        else:
            if sys.platform == "darwin":
                self.qtlib = [f"Qt/lib/{i.replace('libQt6', 'Qt')}.framework" for i in self.qtlib]
                self.lib = [self.macos_pyside_wrappers_lib(i) for i in self.lib]
            else:
                self.lib = [f"{i}.*{self.ext}*" for i in self.lib]
                self.qtlib = [f"Qt/lib/{i}.*{self.ext}*" for i in self.qtlib]
            self.qml = [f"Qt/qml/{i}" for i in self.qml]
            self.translations = [f"Qt/translations/{i}" for i in self.translations]
            self.metatypes = [f"Qt/metatypes/{i}" for i in self.metatypes]
            self.plugins = [f"Qt/plugins/{i}" for i in self.plugins]

        self.typesystems = [f"typesystems/{i}" for i in self.typesystems]
        self.include = [f"include/{i}" for i in self.include]
        self.glue = [f"glue/{i}" for i in self.glue]
        self.doc_glue = [f"doc/{i}" for i in self.doc_glue]

    def macos_pyside_wrappers_lib(self, s):
        if s.startswith("Qt"):
            return f"{s}.*so*"
        else:
            return f"{s}.*{self.ext}*"

    @classmethod
    def get_fields(cls) -> dict[str, Field]:
        return cls.__dataclass_fields__

    @staticmethod
    def get_extension_from_platform(platform: str) -> str:
        if platform == "linux":
            return "so"
        elif platform == "darwin":
            return "dylib"
        elif platform == "win32":
            return "pyd"
        else:
            print(f"Platform '{platform}' not supported. Exiting")
            sys.exit(-1)


# Wheels auxiliary functions to return the ModuleData objects
# for each module that will be included in the wheel.

# PySide wheel
def wheel_files_pyside_essentials() -> list[ModuleData]:
    files = [
        module_QtCore(),
        module_QtGui(),
        module_QtWidgets(),
        module_QtHelp(),
        module_QtNetwork(),
        module_QtConcurrent(),
        module_QtDBus(),
        module_QtDesigner(),
        module_QtOpenGL(),
        module_QtOpenGLWidgets(),
        module_QtPrintSupport(),
        module_QtQml(),
        module_QtQuick(),
        module_QtQuickControls2(),
        module_QtQuickTest(),
        module_QtQuickWidgets(),
        module_QtXml(),
        module_QtTest(),
        module_QtSql(),
        module_QtSvg(),
        module_QtSvgWidgets(),
        module_QtUiTools(),
        # Only for plugins
        module_QtWayland(),
        # there are no bindings for these modules, but their binaries are
        # required for qmlls
        module_QtLanguageServer(),
        module_QtJsonRpc(),
    ]
    return files


# PySide Addons wheel
def wheel_files_pyside_addons() -> list[ModuleData]:
    files = [
        module_Qt3DAnimation(),
        module_Qt3DCore(),
        module_Qt3DExtras(),
        module_Qt3DInput(),
        module_Qt3DLogic(),
        module_Qt3DRender(),
        module_QtAxContainer(),
        module_QtBluetooth(),
        module_QtCharts(),
        module_QtDataVisualization(),
        module_QtGraphs(),
        module_QtGraphsWidgets(),
        module_QtMultimedia(),
        module_QtMultimediaWidgets(),
        module_QtNetworkAuth(),
        module_QtNfc(),
        module_QtPdf(),
        module_QtPdfWidgets(),
        module_QtPositioning(),
        module_QtQuick3D(),
        module_QtRemoteObjects(),
        module_QtScxml(),
        module_QtSensors(),
        module_QtSerialPort(),
        module_QtSerialBus(),
        module_QtSpatialAudio(),
        module_QtStateMachine(),
        module_QtTextToSpeech(),
        module_QtVirtualKeyboard(),
        module_QtWebChannel(),
        module_QtWebEngineCore(),
        module_QtWebEngineQuick(),
        module_QtWebEngineWidgets(),
        module_QtWebSockets(),
        module_QtHttpServer(),
        module_QtLocation(),
        module_QtAsyncio(),
        module_QtWebView(),
        # This is not an actual module, but it's required in order
        # to add the 'Quick' components of the WebView.
        module_QtWebViewQuick(),
    ]
    return files


# Functions that hold the information of all the files that needs
# to be included for the module to work, including Qt libraries,
# typesystems, glue, etc.
def module_QtCore() -> ModuleData:
    # QtCore
    data = ModuleData("Core")

    _typesystems = [
        "common.xml",
        "core_common.xml",
        "typesystem_core_common.xml",
        "typesystem_core_win.xml"
    ]

    data.typesystems.extend(_typesystems)
    data.include.append("*.h")
    if sys.platform == "win32":
        data.qtlib.append("pyside6.*")
        data.extra_files.append("qt.conf")
        data.extra_files.append("rcc.exe")
        data.extra_files.append("qtdiag.exe")
        data.extra_files.append("pyside6.*.lib")
        data.extra_files.append("resources/icudtl.dat")
        from build_scripts.platforms.windows_desktop import msvc_redist
        data.extra_files.extend(msvc_redist)
    else:
        data.lib.append("libpyside6.*")
        data.extra_files.append("Qt/libexec/rcc")
        data.extra_files.append("Qt/libexec/qt.conf")

    # *.py
    data.extra_dirs.append("support")

    # pyside-tools with python backend
    # Including the 'scripts' folder would include all the tools into the
    # PySide6_Essentials wheel. The moment when we add a tool that has a
    # dependency on a module in PySide6_AddOns, then we should split out
    # the following line into individual subfolder and files, to better
    # control which tool goes into which wheel
    data.extra_dirs.append("scripts")

    data.extra_dirs.append("typesystems/glue")

    data.extra_files.append("__feature__.pyi")
    data.extra_files.append("__init__.py")
    data.extra_files.append("_git_pyside_version.py")
    data.extra_files.append("_config.py")
    data.extra_files.append("py.typed")

    # Assistant
    if sys.platform == "darwin":
        data.extra_dirs.append("Assistant.app")
    else:
        data.extra_files.append("assistant*")
    data.translations.append("assistant_*")

    # Linguist
    if sys.platform == "darwin":
        data.extra_dirs.append("Linguist.app")
    else:
        data.extra_files.append("linguist*")
        data.extra_files.append("lconvert*")
    data.translations.append("linguist_*")

    data.extra_files.append("lrelease*")
    data.extra_files.append("lupdate*")

    # General translations
    data.translations.append("qtbase_*")
    data.translations.append("qt_help_*")
    data.translations.append("qt_*")

    # Extra libraries
    data.qtlib.append("libicudata*")
    data.qtlib.append("libicui18n*")
    data.qtlib.append("libicule*")
    data.qtlib.append("libiculx*")
    data.qtlib.append("libicutest*")
    data.qtlib.append("libicutu*")
    data.qtlib.append("libicuuc*")
    data.qtlib.append("libicuio*")

    # add the include folders for libpyside binaries
    # this is useful for downstream cmake projects like QtBridges
    data.include.append("pyside6/*.h")

    return data


def module_QtGui() -> ModuleData:
    data = ModuleData("Gui")
    _typesystems = [
        "gui_common.xml",
        "typesystem_gui_common.xml",
        "typesystem_gui_nativeinterface.xml",
        "typesystem_gui_mac.xml",
        "typesystem_gui_win.xml",
        "typesystem_gui_x11.xml",
        "typesystem_gui_rhi.xml",
        "typesystem_gui_wayland.xml"
    ]

    _metatypes = [
        "qt6eglfsdeviceintegrationprivate_metatypes.json",
        "qt6eglfskmssupportprivate_metatypes.json",
        "qt6kmssupportprivate_metatypes.json",
        "qt6xcbqpaprivate_metatypes.json",
    ]

    _qtlib = [
        "libQt6EglFSDeviceIntegration",
        "libQt6EglFsKmsSupport",
        "libQt6XcbQpa",
    ]

    data.typesystems.extend(_typesystems)
    data.metatypes.extend(_metatypes)
    data.qtlib.extend(_qtlib)

    json_data = get_module_json_data("Gui")
    data.plugins = get_module_plugins(json_data)
    data.extra_files.append("Qt/plugins/platforms/libqeglfs*")

    return data


def module_QtWidgets() -> ModuleData:
    data = ModuleData("Widgets")
    data.typesystems.append("widgets_common.xml")
    data.typesystems.append("typesystem_widgets_common.xml")

    if sys.platform == "win32":
        data.extra_files.append("uic.exe")
    else:
        data.extra_files.append("Qt/libexec/uic")
    json_data = get_module_json_data("Widgets")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtHelp() -> ModuleData:
    data = ModuleData("Help")

    return data


def module_QtNetwork() -> ModuleData:
    data = ModuleData("Network")
    json_data = get_module_json_data("Network")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtBluetooth() -> ModuleData:
    data = ModuleData("Bluetooth")
    data.translations.append("qtconnectivity_*")

    return data


def module_QtConcurrent() -> ModuleData:
    data = ModuleData("Concurrent")

    return data


def module_QtDBus() -> ModuleData:
    data = ModuleData("DBus")

    return data


def module_QtDesigner() -> ModuleData:
    data = ModuleData("Designer")
    data.qtlib.append("libQt6DesignerComponents")
    data.metatypes.append("qt6designercomponentsprivate_metatypes.json")
    json_data = get_module_json_data("Designer")
    data.plugins = get_module_plugins(json_data)
    data.extra_files.append("Qt/plugins/assetimporters/libuip*")

    # Designer
    if sys.platform == "darwin":
        data.extra_dirs.append("Designer.app")
    else:
        data.extra_files.append("designer*")
    data.translations.append("designer_*")

    return data


def module_QtNfc() -> ModuleData:
    data = ModuleData("Nfc")

    return data


def module_QtPdf() -> ModuleData:
    data = ModuleData("Pdf")
    data.qtlib.append("libQt6PdfQuick")

    return data


def module_QtPdfWidgets() -> ModuleData:
    data = ModuleData("PdfWidgets")

    return data


def module_QtPrintSupport() -> ModuleData:
    data = ModuleData("PrintSupport")
    data.typesystems.append("typesystem_printsupport_common.xml")
    json_data = get_module_json_data("PrintSupport")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtQml() -> ModuleData:
    data = ModuleData("Qml")
    json_data = get_module_json_data("Qml")
    data.plugins = get_module_plugins(json_data)
    json_data = get_module_json_data("QmlCompilerPrivate")
    data.plugins += get_module_plugins(json_data)

    _qtlib = [
        "libQt6LabsAnimation",
        "libQt6LabsFolderListModel",
        "libQt6LabsQmlModels*",
        "libQt6LabsSettings",
        "libQt6LabsSharedImage",
        "libQt6LabsWavefrontMesh",
        "libQt6QmlCore",
        "libQt6QmlLocalStorage",
        "libQt6QmlModels",
        "libQt6QmlNetwork",
        "libQt6QmlWorkerScript",
        "libQt6QmlXmlListModel",
        "libQt6QmlCompiler",
        "libQt6QmlMeta",
        "libQt6LabsPlatform",
    ]

    _include = [
        "pysideqml.h",
        "pysideqmlmacros.h",
        "pysideqmlregistertype.h",
    ]

    _metatypes = [
        "qt6labsanimation_metatypes.json",
        "qt6labsfolderlistmodel_metatypes.json",
        "qt6labsqmlmodels_metatypes.json",
        "qt6labssettings_metatypes.json",
        "qt6labssharedimage_metatypes.json",
        "qt6labswavefrontmesh_metatypes.json",
        "qt6packetprotocolprivate_metatypes.json",
        "qt6qmlcompilerprivate_metatypes.json",
        "qt6qmlcompilerplusprivate_metatypes.json",
        "qt6qmlcore_metatypes.json",
        "qt6qmldebugprivate_metatypes.json",
        "qt6qmldomprivate_metatypes.json",
        "qt6qmllintprivate_metatypes.json",
        "qt6qmllocalstorage_metatypes.json",
        "qt6qmlmodels_metatypes.json",
        "qt6qmlworkerscript_metatypes.json",
        "qt6qmlxmllistmodel_metatypes.json",
        "qt6qmlmeta_metatypes.json",
        "qt6labsplatform_metatypes.json",
    ]

    _qml = [
        "Qt/labs/animation",
        "Qt/labs/folderlistmodel",
        "Qt/labs/sharedimage",
        "Qt/labs/wavefrontmesh",
        "Qt/labs/qmlmodels",
        "Qt/labs/platform",
        "Qt/labs/settings",
    ]

    data.lib.append("libpyside6qml")
    json_data = get_module_json_data("Qml")
    data.plugins = get_module_plugins(json_data)
    data.translations.append("qtdeclarative_*")
    if sys.platform == "win32":
        data.extra_files.append("pyside6qml.*.lib")
        data.extra_files.append("pyside6qml.*.dll")
        data.extra_files.append("qml/builtins.qmltypes")
        data.extra_files.append("qml/jsroot.qmltypes")
        data.extra_files.append("qmlimportscanner.exe")
        data.extra_files.append("qmltyperegistrar.exe")
        data.extra_files.append("qmlcachegen.exe")
    else:
        data.extra_files.append("Qt/qml/builtins.qmltypes")
        data.extra_files.append("Qt/qml/jsroot.qmltypes")
        data.extra_files.append("Qt/libexec/qmlimportscanner")
        data.extra_files.append("Qt/libexec/qmltyperegistrar")
        data.extra_files.append("Qt/libexec/qmlcachegen")

    data.qtlib.extend(_qtlib)
    data.include.extend(_include)
    data.metatypes.extend(_metatypes)
    data.qml.extend(_qml)

    data.extra_files.append("qmllint*")
    # adds qmllint plugins
    json_data_qmllint = get_module_json_data("QmlCompiler")
    qml_lint_plugins = get_module_plugins(json_data_qmllint)
    data.plugins += qml_lint_plugins

    data.extra_files.append("qmlformat*")
    data.extra_files.append("qmlls*")

    return data


def module_QtQuick() -> ModuleData:
    data = ModuleData("Quick")
    _metatypes = [
        "qt6quickcontrolstestutilsprivate_metatypes.json",
        "qt6quickdialogs2_metatypes.json",
        "qt6quickdialogs2quickimpl_metatypes.json",
        "qt6quickdialogs2utils_metatypes.json",
        "qt6quickeffectsprivate_metatypes.json",
        "qt6quicketest_metatypes.json",
        "qt6quicketestutilsprivate_metatypes.json",
        "qt6quicklayouts_metatypes.json",
        "qt6quickparticlesprivate_metatypes.json",
        "qt6quickshapesprivate_metatypes.json",
        "qt6quicktemplates2_metatypes.json",
        "qt6quicktest_metatypes.json",
        "qt6quicktestutilsprivate_metatypes.json",
        "qt6quicktimeline_metatypes.json",
        "qt6quickvectorimage_metatypes.json",
        "qt6quickvectorimagegeneratorprivate_metatypes.json",
    ]
    _qtlib = [
        "libQt6QuickEffects",
        "libQt6QuickDialogs2",
        "libQt6QuickDialogs2QuickImpl",
        "libQt6QuickDialogs2Utils",
        "libQt6QuickLayouts",
        "libQt6QuickParticles",
        "libQt6QuickShapes",
        "libQt6QuickTemplates2",
        "libQt6QuickTest",
        "libQt6QuickTimeline",
        "libQt6QuickTimelineBlendTrees",
        "libQt6QuickVectorImage",
        "libQt6QuickVectorImageGenerator",
        "libQt6QuickVectorImageHelpers"
    ]

    data.qtlib.extend(_qtlib)
    data.metatypes.extend(_metatypes)
    json_data = get_module_json_data("Quick")
    data.plugins = get_module_plugins(json_data)

    # This includes the vectorimageformats plugin
    json_data_vector = get_module_json_data("QuickVectorImageGeneratorPrivate")
    vector_plugins = get_module_plugins(json_data_vector)
    if vector_plugins:
        data.plugins += vector_plugins

    data.extra_files.append("svgtoqml*")

    return data


def module_QtQuickControls2() -> ModuleData:
    data = ModuleData("QuickControls2")
    data.qtlib.append("libQt6QuickControls2")
    data.qtlib.append("libQt6QuickControls2Basic")
    data.qtlib.append("libQt6QuickControls2BasicStyleImpl")
    data.qtlib.append("libQt6QuickControls2Fusion")
    data.qtlib.append("libQt6QuickControls2FusionStyleImpl")
    data.qtlib.append("libQt6QuickControls2Imagine")
    data.qtlib.append("libQt6QuickControls2ImagineStyleImpl")
    data.qtlib.append("libQt6QuickControls2Impl")
    data.qtlib.append("libQt6QuickControls2Material")
    data.qtlib.append("libQt6QuickControls2MaterialStyleImpl")
    data.qtlib.append("libQt6QuickControls2Universal")
    data.qtlib.append("libQt6QuickControls2UniversalStyleImpl")
    # FluentWinUI3 Style is available for all platforms, even if it
    # was originally intended for Windows.
    data.qtlib.append("libQt6QuickControls2FluentWinUI3StyleImpl")
    if sys.platform == "win32":
        data.qtlib.append("libQt6QuickControls2WindowsStyleImpl")
    elif sys.platform == "darwin":
        data.qtlib.append("libQt6QuickControls2IOSStyleImpl")
        data.qtlib.append("libQt6QuickControls2MacOSStyleImpl")

    data.metatypes.append("qt6quickcontrols2impl_metatypes.json")

    return data


def module_QtQuickTest() -> ModuleData:
    data = ModuleData("QuickTest")

    return data


def module_QtQuickWidgets() -> ModuleData:
    data = ModuleData("QuickWidgets")
    return data


def module_QtXml() -> ModuleData:
    data = ModuleData("Xml")
    return data


def module_QtTest() -> ModuleData:
    data = ModuleData("Test")
    return data


def module_QtSql() -> ModuleData:
    data = ModuleData("Sql")
    json_data = get_module_json_data("Sql")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtSvg() -> ModuleData:
    data = ModuleData("Svg")

    return data


def module_QtSvgWidgets() -> ModuleData:
    data = ModuleData("SvgWidgets")

    return data


def module_QtTextToSpeech() -> ModuleData:
    data = ModuleData("TextToSpeech")
    json_data = get_module_json_data("TextToSpeech")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtUiTools() -> ModuleData:
    data = ModuleData("UiTools")

    return data


def module_QtWayland() -> ModuleData:
    data = ModuleData("Wayland")

    _qtlib = [
        "libQt6WaylandClient",
        "libQt6WaylandCompositor",
        "libQt6WaylandEglClientHwIntegration",
        "libQt6WaylandEglCompositorHwIntegration",
        "libQt6WlShellIntegration",
    ]

    _metatypes = [
        "qt6waylandclient_metatypes.json",
        "qt6waylandeglclienthwintegrationprivate_metatypes.json",
        "qt6wlshellintegrationprivate_metatypes.json",
    ]

    data.qtlib.extend(_qtlib)
    data.metatypes.extend(_metatypes)
    json_data = get_module_json_data("WaylandClient")
    data.plugins = get_module_plugins(json_data)
    json_data = get_module_json_data("WaylandCompositor")
    data.plugins += get_module_plugins(json_data)
    return data


def module_Qt3DCore() -> ModuleData:
    data = ModuleData("3DCore", qml=["Qt3D/Core"])

    return data


def module_Qt3DAnimation() -> ModuleData:
    data = ModuleData("3DAnimation", qml=["Qt3D/Animation"])

    return data


def module_Qt3DExtras() -> ModuleData:
    data = ModuleData("3DExtras", qml=["Qt3D/Extras"])

    return data


def module_Qt3DInput() -> ModuleData:
    data = ModuleData("3DInput", qml=["Qt3D/Input"])
    json_data = get_module_json_data("3DInput")
    data.plugins = get_module_plugins(json_data)

    return data


def module_Qt3DLogic() -> ModuleData:
    data = ModuleData("3DLogic", qml=["Qt3D/Logic"])
    json_data = get_module_json_data("3DLogic")
    data.plugins = get_module_plugins(json_data)

    return data


def module_Qt3DRender() -> ModuleData:
    data = ModuleData("3DRender", qml=["Qt3D/Render"])
    json_data = get_module_json_data("3DRender")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtQuick3D() -> ModuleData:
    data = ModuleData("Quick3D")

    _qtlib = [
        "libQt6Quick3DAssetImport",
        "libQt6Quick3DAssetUtils",
        "libQt6Quick3DEffects",
        "libQt6Quick3DGlslParser",
        "libQt6Quick3DHelpers",
        "libQt6Quick3DHelpersImpl",
        "libQt6Quick3DIblBaker",
        "libQt6Quick3DParticleEffects",
        "libQt6Quick3DParticles",
        "libQt6Quick3DPhysics",
        "libQt6Quick3DPhysicsHelpers",
        "libQt6Quick3DRuntimeRender",
        "libQt6Quick3DSpatialAudio",
        "libQt6Quick3DUtils",
        "libQt6ShaderTools",
        "libQt63DQuick",
        "libQt63DQuickAnimation",
        "libQt63DQuickExtras",
        "libQt63DQuickLogic",
        "libQt63DQuickInput",
        "libQt63DQuickRender",
        "libQt63DQuickScene2D",
        "libQt63DQuickScene3D",
        "libQt6Quick3DXr",
    ]

    _metatypes = [
        "qt63dquick_metatypes.json",
        "qt63dquickanimation_metatypes.json",
        "qt63dquickextras_metatypes.json",
        "qt63dquickinput_metatypes.json",
        "qt63dquickrender_metatypes.json",
        "qt63dquickscene2d_metatypes.json",
        "qt6quick3dassetimport_metatypes.json",
        "qt6quick3dassetutils_metatypes.json",
        "qt6quick3deffects_metatypes.json",
        "qt6quick3dglslparserprivate_metatypes.json",
        "qt6quick3dhelpers_metatypes.json",
        "qt6quick3diblbaker_metatypes.json",
        "qt6quick3dparticleeffects_metatypes.json",
        "qt6quick3dparticles_metatypes.json",
        "qt6quick3druntimerender_metatypes.json",
        "qt6quick3dutils_metatypes.json",
        "qt6shadertools_metatypes.json",
        "qt6quick3dxr_metatypes.json"
    ]

    json_data = get_module_json_data("Quick3DAssetImport")
    data.plugins = get_module_plugins(json_data)
    data.qtlib.extend(_qtlib)
    data.metatypes.extend(_metatypes)
    data.extra_files.append("Qt/plugins/assetimporters/libassimp*")
    data.extra_files.append("qsb*")
    data.extra_files.append("balsam*")

    # Adding GraphicalEffects files
    data.qml.append("Qt5Compat/GraphicalEffects")

    return data


def module_QtAxContainer() -> ModuleData:
    data = ModuleData("AxContainer")
    if sys.platform == "win32":
        data.metatypes.append("qt6axbaseprivate_metatypes.json")
        data.metatypes.append("qt6axserver_metatypes.json")

    return data


def module_QtWebEngineCore() -> ModuleData:
    data = ModuleData("WebEngineCore", qml=["QtWebEngine"])
    data.translations.append("qtwebengine_locales/*")
    data.translations.append("qtwebengine_*")
    data.extra_dirs.append("Qt/resources")
    if sys.platform == "win32":
        data.extra_files.append("resources/qtwebengine*.pak")
        data.extra_files.append("resources/v8_context_snapshot*.*")
        data.extra_files.append("QtWebEngineProcess.exe")
    else:
        data.extra_files.append("Qt/libexec/QtWebEngineProcess")

    return data


def module_QtWebEngineWidgets() -> ModuleData:
    data = ModuleData("WebEngineWidgets")

    return data


def module_QtWebEngineQuick() -> ModuleData:
    data = ModuleData("WebEngineQuick")
    data.qtlib.append("libQt6WebEngineQuickDelegatesQml")
    data.metatypes.append("qt6webenginequickdelegatesqml_metatypes.json")

    return data


def module_QtCharts() -> ModuleData:
    data = ModuleData("Charts")
    data.qtlib.append("libQt6ChartsQml")
    data.metatypes.append("qt6chartsqml_metatypes.json")

    return data


def module_QtDataVisualization() -> ModuleData:
    data = ModuleData("DataVisualization")
    data.qtlib.append("libQt6DataVisualizationQml")
    data.metatypes.append("qt6datavisualizationqml_metatypes.json")
    data.typesystems.append("datavisualization_common.xml")

    return data


def module_QtGraphs() -> ModuleData:
    data = ModuleData("Graphs")

    return data


def module_QtGraphsWidgets() -> ModuleData:
    data = ModuleData("GraphsWidgets")

    return data


def module_QtMultimedia() -> ModuleData:
    data = ModuleData("Multimedia")
    data.qtlib.append("libQt6MultimediaQuick")
    data.metatypes.append("qt6multimediaquickprivate_metatypes.json")

    json_data = get_module_json_data("Multimedia")
    data.translations.append("qtmultimedia_*")
    data.plugins = get_module_plugins(json_data)

    linux_multimedia_libs = [f"Qt/lib/lib{lib}*.so*" for lib in _multimedia_libs]
    linux_multimedia_libs.append("Qt/lib/libQt6FFmpegStub*.so*")
    platform_files = {
        "win32": [f"{lib}-*.dll" for lib in _multimedia_libs],
        "darwin": [f"Qt/lib/lib{lib}.*.dylib" for lib in _multimedia_libs],
        "linux": linux_multimedia_libs}
    extra_files = platform_files.get(sys.platform, [])
    data.extra_files.extend(extra_files)

    return data


def module_QtMultimediaWidgets() -> ModuleData:
    data = ModuleData("MultimediaWidgets")

    return data


def module_QtNetworkAuth() -> ModuleData:
    data = ModuleData("NetworkAuth")

    return data


def module_QtPositioning() -> ModuleData:
    data = ModuleData("Positioning")
    data.qtlib.append("libQt6PositioningQuick")
    data.metatypes.append("qt6positioningquick_metatypes.json")
    json_data = get_module_json_data("Positioning")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtRemoteObjects() -> ModuleData:
    data = ModuleData("RemoteObjects")
    data.qtlib.append("libQt6RemoteObjectsQml")
    data.metatypes.append("qt6remoteobjectsqml_metatypes.json")

    return data


def module_QtSensors() -> ModuleData:
    data = ModuleData("Sensors")
    data.qtlib.append("libQt6SensorsQuick")
    data.metatypes.append("qt6sensorsquick_metatypes.json")
    json_data = get_module_json_data("Sensors")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtSerialPort() -> ModuleData:
    data = ModuleData("SerialPort")
    data.translations.append("qtserialport_*")

    return data


def module_QtSpatialAudio() -> ModuleData:
    data = ModuleData("SpatialAudio")
    data.metatypes.append("qt6spatialaudio_metatypes.json")

    return data


def module_QtStateMachine() -> ModuleData:
    data = ModuleData("StateMachine")
    data.qtlib.append("libQt6StateMachineQml")
    data.metatypes.append("qt6statemachineqml_metatypes.json")

    return data


def module_QtScxml() -> ModuleData:
    data = ModuleData("Scxml")
    data.qtlib.append("libQt6ScxmlQml")
    data.metatypes.append("qt6scxmlqml_metatypes.json")
    json_data = get_module_json_data("Scxml")
    data.plugins = get_module_plugins(json_data)

    return data


def module_QtWebChannel() -> ModuleData:
    data = ModuleData("WebChannel")
    data.qtlib.append("libQt6WebChannelQuick")

    return data


def module_QtWebSockets() -> ModuleData:
    data = ModuleData("WebSockets")
    data.translations.append("qtwebsockets_*")

    return data


def module_QtOpenGL() -> ModuleData:
    data = ModuleData("OpenGL")
    _typesystems = [
        "opengl_common.xml",
        "typesystem_glgeti_v_includes.xml",
        "typesystem_glgeti_v_modifications.xml",
        "typesystem_glgetv_includes.xml",
        "typesystem_glgetv_modifications.xml",
        "typesystem_opengl_modifications1_0.xml",
        "typesystem_opengl_modifications1_0_compat.xml",
        "typesystem_opengl_modifications1_1.xml",
        "typesystem_opengl_modifications1_1_compat.xml",
        "typesystem_opengl_modifications1_2_compat.xml",
        "typesystem_opengl_modifications1_3_compat.xml",
        "typesystem_opengl_modifications1_4.xml",
        "typesystem_opengl_modifications1_4_compat.xml",
        "typesystem_opengl_modifications2_0.xml",
        "typesystem_opengl_modifications2_0_compat.xml",
        "typesystem_opengl_modifications2_1.xml",
        "typesystem_opengl_modifications3_0.xml",
        "typesystem_opengl_modifications3_3.xml",
        "typesystem_opengl_modifications3_3a.xml",
        "typesystem_opengl_modifications4_0.xml",
        "typesystem_opengl_modifications4_1.xml",
        "typesystem_opengl_modifications4_3.xml",
        "typesystem_opengl_modifications4_4.xml",
        "typesystem_opengl_modifications4_4_core.xml",
        "typesystem_opengl_modifications4_5.xml",
        "typesystem_opengl_modifications4_5_core.xml",
        "typesystem_opengl_modifications_va.xml",
    ]

    data.typesystems.extend(_typesystems)
    if sys.platform == "win32":
        data.extra_files.append("opengl32*.dll")

    return data


def module_QtOpenGLWidgets() -> ModuleData:
    data = ModuleData("OpenGLWidgets")
    return data


def module_QtSerialBus() -> ModuleData:
    data = ModuleData("SerialBus")
    json_data = get_module_json_data("SerialBus")
    data.plugins = get_module_plugins(json_data)
    return data


def module_QtVirtualKeyboard() -> ModuleData:
    data = ModuleData("VirtualKeyboard")
    data.plugins.append("virtualkeyboard")
    data.qtlib.append("libQt6VirtualKeyboardSettings")
    data.qtlib.append("libQt6VirtualKeyboardQml")

    return data


def module_QtHttpServer() -> ModuleData:
    data = ModuleData("HttpServer")
    return data


def module_QtLanguageServer() -> ModuleData:
    data = ModuleData("LanguageServer")
    data.metatypes.append("qt6languageserverprivate_metatypes.json")
    return data


def module_QtJsonRpc() -> ModuleData:
    data = ModuleData("JsonRpc")
    data.metatypes.append("qt6jsonrpcprivate_metatypes.json")
    return data


def module_QtLocation() -> ModuleData:
    data = ModuleData("Location")
    json_data = get_module_json_data("Location")
    data.plugins = get_module_plugins(json_data)
    data.translations.append("qtlocation_*")
    return data


def module_QtAsyncio() -> ModuleData:
    data = ModuleData("Asyncio")
    data.extra_dirs.append("QtAsyncio")
    return data


def module_QtWebView() -> ModuleData:
    data = ModuleData("WebView")
    json_data = get_module_json_data("WebView")
    data.plugins = get_module_plugins(json_data)
    return data


def module_QtWebViewQuick() -> ModuleData:
    data = ModuleData("WebViewQuick")
    return data



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from PySide6.QtWidgets import QApplication
from bookwindow import BookWindow

if __name__ == "__main__":
    app = QApplication([])

    window = BookWindow()
    window.resize(800, 600)
    window.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/richtext/textedit example from Qt v6.x"""

import sys
from argparse import ArgumentParser, RawTextHelpFormatter

from PySide6.QtCore import QCoreApplication, qVersion
from PySide6.QtWidgets import QApplication

from textedit import TextEdit

import textedit_rc  # noqa: F401


if __name__ == '__main__':
    argument_parser = ArgumentParser(description='Rich Text Example',
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("file", help="File",
                                 nargs='?', type=str)
    options = argument_parser.parse_args()

    app = QApplication(sys.argv)
    QCoreApplication.setOrganizationName("QtProject")
    QCoreApplication.setApplicationName("Rich Text")
    QCoreApplication.setApplicationVersion(qVersion())

    mw = TextEdit()

    available_geometry = mw.screen().availableGeometry()
    mw.resize((available_geometry.width() * 2) / 3,
              (available_geometry.height() * 2) / 3)
    mw.move((available_geometry.width() - mw.width()) / 2,
            (available_geometry.height() - mw.height()) / 2)

    file = options.file if options.file else ":/example.html"
    if not mw.load(file):
        mw.file_new()

    mw.show()
    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtDataVisualization'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtCore import QTimer
from PySide6.QtDataVisualization import (Q3DBars, QBar3DSeries, QBarDataItem,
                                         QBarDataProxy, QCategory3DAxis,
                                         QValue3DAxis, QValue3DAxisFormatter,
                                         qDefaultSurfaceFormat)


def dataToBarDataRow(data):
    result = []
    for d in data:
        result.append(QBarDataItem(d))
    return result


def dataToBarDataArray(data):
    result = []
    for row in data:
        result.append(dataToBarDataRow(row))
    return result


class QtDataVisualizationTestCase(UsesQApplication):
    '''Tests related to QtDataVisualization'''

    def testBars(self):
        self.bars = Q3DBars()
        self.columnAxis = QCategory3DAxis()
        self.columnAxis.setTitle('Columns')
        self.columnAxis.setTitleVisible(True)
        self.columnAxis.setLabels(['Column1', 'Column2'])

        self.rowAxis = QCategory3DAxis()
        self.rowAxis.setTitle('Rows')
        self.rowAxis.setTitleVisible(True)
        self.rowAxis.setLabels(['Row1', 'Row2'])

        self.valueAxis = QValue3DAxis()
        self.valueAxis.setTitle('Values')
        self.valueAxis.setTitleVisible(True)
        self.valueAxis.setRange(0, 5)

        self.bars.setRowAxis(self.rowAxis)
        self.bars.setColumnAxis(self.columnAxis)
        self.bars.setValueAxis(self.valueAxis)

        self.series = QBar3DSeries()
        self.arrayData = [[1, 2], [3, 4]]
        self.series.dataProxy().addRows(dataToBarDataArray(self.arrayData))

        self.bars.setPrimarySeries(self.series)

        self.bars.show()
        QTimer.singleShot(500, self.app.quit)
        self.app.exec()

    def testBarDataProxy(self):
        '''PSYSIDE-1438, crashes in QBarDataProxy.addRow()'''
        items = [QBarDataItem(v) for v in [1.0, 2.0]]
        data_proxy = QBarDataProxy()
        data_proxy.addRow(items)
        data_proxy.addRow(items, 'bla')
        data_proxy.insertRow(0, items)
        data_proxy.insertRow(0, items, 'bla')
        data_proxy.setRow(0, items)
        data_proxy.setRow(0, items, 'bla')
        self.assertTrue(data_proxy.rowCount(), 4)

    def testDefaultSurfaceFormat(self):
        format = qDefaultSurfaceFormat(True)
        print(format)

    def testQValue3DAxisFormatter(self):
        """PYSIDE-2025: Test the added setters of QValue3DAxisFormatter."""
        formatter = QValue3DAxisFormatter()
        float_values = [float(10)]
        formatter.setGridPositions(float_values)
        self.assertEqual(formatter.gridPositions(), float_values)

        formatter.setLabelPositions(float_values)
        self.assertEqual(formatter.labelPositions(), float_values)

        label_strings = ["bla"]
        formatter.setLabelStrings(label_strings)
        self.assertEqual(formatter.labelStrings(), label_strings)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import shutil
import zipfile
from pathlib import Path

from pythonforandroid.logger import info
from pythonforandroid.recipe import PythonRecipe


class ShibokenRecipe(PythonRecipe):
    version = '{{ version }}'
    wheel_path = '{{ wheel_path }}'

    call_hostpython_via_targetpython = False
    install_in_hostpython = False

    def build_arch(self, arch):
        ''' Unzip the wheel and copy into site-packages of target'''
        info('Installing {} into site-packages'.format(self.name))
        with zipfile.ZipFile(self.wheel_path, 'r') as zip_ref:
            info('Unzip wheels and copy into {}'.format(self.ctx.get_python_install_dir(arch.arch)))
            zip_ref.extractall(self.ctx.get_python_install_dir(arch.arch))

        lib_dir = Path(f"{self.ctx.get_python_install_dir(arch.arch)}/shiboken6")
        shutil.copyfile(lib_dir / "libshiboken6.abi3.so",
                        Path(self.ctx.get_libs_dir(arch.arch)) / "libshiboken6.abi3.so")


recipe = ShibokenRecipe()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import SIGNAL
from PySide6.QtWidgets import QPushButton, QApplication


class Test (QApplication):
    def __init__(self, argv):
        super().__init__(argv)
        self._called = False

    def called(self):
        self._called = True


class QApplicationSignalsTest(unittest.TestCase):
    def testQuit(self):
        app = Test([])
        button = QPushButton("BUTTON")
        app.connect(button, SIGNAL("clicked()"), app.called)
        button.click()
        self.assertTrue(app._called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

""" Unittest for bug #576 """
""" http://bugs.openbossa.org/show_bug.cgi?id=576 """

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject
from PySide6.QtWidgets import QApplication, QPushButton, QWidget


class Bug576(unittest.TestCase):
    def onButtonDestroyed(self, button):
        self._destroyed = True

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testWidgetParent(self):
        self._destroyed = False
        app = QApplication(sys.argv)  # noqa: F841
        w = QWidget()

        b = QPushButton("test")
        b.destroyed[QObject].connect(self.onButtonDestroyed)
        base_ref_count = sys.getrefcount(b)
        b.setParent(w)
        self.assertEqual(sys.getrefcount(b), base_ref_count + 1)
        b.parent()
        self.assertEqual(sys.getrefcount(b), base_ref_count + 1)
        b.setParent(None)
        self.assertEqual(sys.getrefcount(b), base_ref_count)
        del b
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertTrue(self._destroyed)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QHBoxLayout, QHeaderView, QLabel, QLineEdit,
    QListWidget, QListWidgetItem, QMainWindow, QMenu,
    QMenuBar, QPushButton, QSizePolicy, QSpacerItem,
    QSpinBox, QStatusBar, QTreeView, QVBoxLayout,
    QWidget)
import rc_modbusclient

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(601, 378)
        MainWindow.setMaximumSize(QSize(16777215, 1000))
        self.actionConnect = QAction(MainWindow)
        self.actionConnect.setObjectName(u"actionConnect")
        icon = QIcon()
        icon.addFile(u":/images/connect.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionConnect.setIcon(icon)
        self.actionDisconnect = QAction(MainWindow)
        self.actionDisconnect.setObjectName(u"actionDisconnect")
        icon1 = QIcon()
        icon1.addFile(u":/images/disconnect.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionDisconnect.setIcon(icon1)
        self.actionExit = QAction(MainWindow)
        self.actionExit.setObjectName(u"actionExit")
        icon2 = QIcon()
        icon2.addFile(u":/images/application-exit.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionExit.setIcon(icon2)
        self.actionOptions = QAction(MainWindow)
        self.actionOptions.setObjectName(u"actionOptions")
        icon3 = QIcon()
        icon3.addFile(u":/images/settings.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionOptions.setIcon(icon3)
        self.centralWidget = QWidget(MainWindow)
        self.centralWidget.setObjectName(u"centralWidget")
        self.verticalLayout = QVBoxLayout(self.centralWidget)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_27 = QLabel(self.centralWidget)
        self.label_27.setObjectName(u"label_27")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_27.sizePolicy().hasHeightForWidth())
        self.label_27.setSizePolicy(sizePolicy)

        self.gridLayout.addWidget(self.label_27, 0, 5, 1, 1)

        self.connectButton = QPushButton(self.centralWidget)
        self.connectButton.setObjectName(u"connectButton")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.connectButton.sizePolicy().hasHeightForWidth())
        self.connectButton.setSizePolicy(sizePolicy1)
        self.connectButton.setCheckable(False)
        self.connectButton.setAutoDefault(False)

        self.gridLayout.addWidget(self.connectButton, 0, 7, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer, 0, 4, 1, 1)

        self.serverEdit = QSpinBox(self.centralWidget)
        self.serverEdit.setObjectName(u"serverEdit")
        sizePolicy1.setHeightForWidth(self.serverEdit.sizePolicy().hasHeightForWidth())
        self.serverEdit.setSizePolicy(sizePolicy1)
        self.serverEdit.setMinimum(1)
        self.serverEdit.setMaximum(247)

        self.gridLayout.addWidget(self.serverEdit, 0, 6, 1, 1)

        self.connectType = QComboBox(self.centralWidget)
        self.connectType.addItem("")
        self.connectType.addItem("")
        self.connectType.setObjectName(u"connectType")

        self.gridLayout.addWidget(self.connectType, 0, 1, 1, 1)

        self.label_2 = QLabel(self.centralWidget)
        self.label_2.setObjectName(u"label_2")
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)

        self.gridLayout.addWidget(self.label_2, 0, 2, 1, 1)

        self.label = QLabel(self.centralWidget)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.portEdit = QLineEdit(self.centralWidget)
        self.portEdit.setObjectName(u"portEdit")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.portEdit.sizePolicy().hasHeightForWidth())
        self.portEdit.setSizePolicy(sizePolicy2)

        self.gridLayout.addWidget(self.portEdit, 0, 3, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.groupBox_2 = QGroupBox(self.centralWidget)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setMinimumSize(QSize(250, 0))
        self.gridLayout_3 = QGridLayout(self.groupBox_2)
        self.gridLayout_3.setSpacing(6)
        self.gridLayout_3.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_3.addWidget(self.label_4, 0, 0, 1, 1)

        self.readAddress = QSpinBox(self.groupBox_2)
        self.readAddress.setObjectName(u"readAddress")
        self.readAddress.setMaximum(9)

        self.gridLayout_3.addWidget(self.readAddress, 0, 1, 1, 1)

        self.label_5 = QLabel(self.groupBox_2)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_3.addWidget(self.label_5, 1, 0, 1, 1)

        self.readSize = QComboBox(self.groupBox_2)
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.addItem("")
        self.readSize.setObjectName(u"readSize")

        self.gridLayout_3.addWidget(self.readSize, 1, 1, 1, 1)

        self.label_9 = QLabel(self.groupBox_2)
        self.label_9.setObjectName(u"label_9")

        self.gridLayout_3.addWidget(self.label_9, 2, 0, 1, 1)

        self.readValue = QListWidget(self.groupBox_2)
        self.readValue.setObjectName(u"readValue")
        self.readValue.setMinimumSize(QSize(0, 0))

        self.gridLayout_3.addWidget(self.readValue, 3, 0, 1, 2)


        self.horizontalLayout_2.addWidget(self.groupBox_2)

        self.writeGroupBox = QGroupBox(self.centralWidget)
        self.writeGroupBox.setObjectName(u"writeGroupBox")
        self.writeGroupBox.setMinimumSize(QSize(225, 0))
        self.gridLayout_2 = QGridLayout(self.writeGroupBox)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_7 = QLabel(self.writeGroupBox)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout_2.addWidget(self.label_7, 0, 0, 1, 1)

        self.writeValueTable = QTreeView(self.writeGroupBox)
        self.writeValueTable.setObjectName(u"writeValueTable")
        self.writeValueTable.setProperty(u"showDropIndicator", True)
        self.writeValueTable.setAlternatingRowColors(True)
        self.writeValueTable.setRootIsDecorated(False)
        self.writeValueTable.setUniformRowHeights(True)
        self.writeValueTable.setItemsExpandable(False)
        self.writeValueTable.setExpandsOnDoubleClick(False)
        self.writeValueTable.header().setVisible(True)

        self.gridLayout_2.addWidget(self.writeValueTable, 3, 0, 1, 2)

        self.writeAddress = QSpinBox(self.writeGroupBox)
        self.writeAddress.setObjectName(u"writeAddress")
        self.writeAddress.setMaximum(9)

        self.gridLayout_2.addWidget(self.writeAddress, 0, 1, 1, 1)

        self.label_8 = QLabel(self.writeGroupBox)
        self.label_8.setObjectName(u"label_8")

        self.gridLayout_2.addWidget(self.label_8, 1, 0, 1, 1)

        self.writeSize = QComboBox(self.writeGroupBox)
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.addItem("")
        self.writeSize.setObjectName(u"writeSize")

        self.gridLayout_2.addWidget(self.writeSize, 1, 1, 1, 1)

        self.label_3 = QLabel(self.writeGroupBox)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout_2.addWidget(self.label_3, 2, 0, 1, 1)


        self.horizontalLayout_2.addWidget(self.writeGroupBox)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label_6 = QLabel(self.centralWidget)
        self.label_6.setObjectName(u"label_6")

        self.horizontalLayout.addWidget(self.label_6)

        self.writeTable = QComboBox(self.centralWidget)
        self.writeTable.setObjectName(u"writeTable")

        self.horizontalLayout.addWidget(self.writeTable)

        self.horizontalSpacer_2 = QSpacerItem(13, 17, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.readButton = QPushButton(self.centralWidget)
        self.readButton.setObjectName(u"readButton")
        sizePolicy1.setHeightForWidth(self.readButton.sizePolicy().hasHeightForWidth())
        self.readButton.setSizePolicy(sizePolicy1)

        self.horizontalLayout.addWidget(self.readButton)

        self.writeButton = QPushButton(self.centralWidget)
        self.writeButton.setObjectName(u"writeButton")

        self.horizontalLayout.addWidget(self.writeButton)

        self.readWriteButton = QPushButton(self.centralWidget)
        self.readWriteButton.setObjectName(u"readWriteButton")
        self.readWriteButton.setEnabled(False)

        self.horizontalLayout.addWidget(self.readWriteButton)


        self.verticalLayout.addLayout(self.horizontalLayout)

        MainWindow.setCentralWidget(self.centralWidget)
        self.statusBar = QStatusBar(MainWindow)
        self.statusBar.setObjectName(u"statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.menuBar = QMenuBar(MainWindow)
        self.menuBar.setObjectName(u"menuBar")
        self.menuBar.setGeometry(QRect(0, 0, 601, 26))
        self.menuDevice = QMenu(self.menuBar)
        self.menuDevice.setObjectName(u"menuDevice")
        self.menuToo_ls = QMenu(self.menuBar)
        self.menuToo_ls.setObjectName(u"menuToo_ls")
        MainWindow.setMenuBar(self.menuBar)
        QWidget.setTabOrder(self.connectType, self.portEdit)
        QWidget.setTabOrder(self.portEdit, self.serverEdit)
        QWidget.setTabOrder(self.serverEdit, self.connectButton)
        QWidget.setTabOrder(self.connectButton, self.readAddress)
        QWidget.setTabOrder(self.readAddress, self.readSize)
        QWidget.setTabOrder(self.readSize, self.readValue)
        QWidget.setTabOrder(self.readValue, self.writeAddress)
        QWidget.setTabOrder(self.writeAddress, self.writeSize)
        QWidget.setTabOrder(self.writeSize, self.writeValueTable)
        QWidget.setTabOrder(self.writeValueTable, self.writeTable)
        QWidget.setTabOrder(self.writeTable, self.readButton)
        QWidget.setTabOrder(self.readButton, self.writeButton)
        QWidget.setTabOrder(self.writeButton, self.readWriteButton)

        self.menuBar.addAction(self.menuDevice.menuAction())
        self.menuBar.addAction(self.menuToo_ls.menuAction())
        self.menuDevice.addAction(self.actionConnect)
        self.menuDevice.addAction(self.actionDisconnect)
        self.menuDevice.addSeparator()
        self.menuDevice.addAction(self.actionExit)
        self.menuToo_ls.addAction(self.actionOptions)

        self.retranslateUi(MainWindow)

        self.connectButton.setDefault(True)
        self.readSize.setCurrentIndex(9)
        self.writeSize.setCurrentIndex(9)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Modbus Client Example", None))
        self.actionConnect.setText(QCoreApplication.translate("MainWindow", u"&Connect", None))
        self.actionDisconnect.setText(QCoreApplication.translate("MainWindow", u"&Disconnect", None))
        self.actionExit.setText(QCoreApplication.translate("MainWindow", u"&Quit", None))
        self.actionOptions.setText(QCoreApplication.translate("MainWindow", u"&Options", None))
        self.label_27.setText(QCoreApplication.translate("MainWindow", u"Server Address:", None))
        self.connectButton.setText(QCoreApplication.translate("MainWindow", u"Connect", None))
        self.connectType.setItemText(0, QCoreApplication.translate("MainWindow", u"Serial", None))
        self.connectType.setItemText(1, QCoreApplication.translate("MainWindow", u"TCP", None))

        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Port:", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Connection type:", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("MainWindow", u"Read", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Start address:", None))
        self.label_5.setText(QCoreApplication.translate("MainWindow", u"Number of values:", None))
        self.readSize.setItemText(0, QCoreApplication.translate("MainWindow", u"1", None))
        self.readSize.setItemText(1, QCoreApplication.translate("MainWindow", u"2", None))
        self.readSize.setItemText(2, QCoreApplication.translate("MainWindow", u"3", None))
        self.readSize.setItemText(3, QCoreApplication.translate("MainWindow", u"4", None))
        self.readSize.setItemText(4, QCoreApplication.translate("MainWindow", u"5", None))
        self.readSize.setItemText(5, QCoreApplication.translate("MainWindow", u"6", None))
        self.readSize.setItemText(6, QCoreApplication.translate("MainWindow", u"7", None))
        self.readSize.setItemText(7, QCoreApplication.translate("MainWindow", u"8", None))
        self.readSize.setItemText(8, QCoreApplication.translate("MainWindow", u"9", None))
        self.readSize.setItemText(9, QCoreApplication.translate("MainWindow", u"10", None))

        self.label_9.setText(QCoreApplication.translate("MainWindow", u"Result:", None))
        self.writeGroupBox.setTitle(QCoreApplication.translate("MainWindow", u"Write", None))
        self.label_7.setText(QCoreApplication.translate("MainWindow", u"Start address:", None))
        self.label_8.setText(QCoreApplication.translate("MainWindow", u"Number of values:", None))
        self.writeSize.setItemText(0, QCoreApplication.translate("MainWindow", u"1", None))
        self.writeSize.setItemText(1, QCoreApplication.translate("MainWindow", u"2", None))
        self.writeSize.setItemText(2, QCoreApplication.translate("MainWindow", u"3", None))
        self.writeSize.setItemText(3, QCoreApplication.translate("MainWindow", u"4", None))
        self.writeSize.setItemText(4, QCoreApplication.translate("MainWindow", u"5", None))
        self.writeSize.setItemText(5, QCoreApplication.translate("MainWindow", u"6", None))
        self.writeSize.setItemText(6, QCoreApplication.translate("MainWindow", u"7", None))
        self.writeSize.setItemText(7, QCoreApplication.translate("MainWindow", u"8", None))
        self.writeSize.setItemText(8, QCoreApplication.translate("MainWindow", u"9", None))
        self.writeSize.setItemText(9, QCoreApplication.translate("MainWindow", u"10", None))

        self.label_3.setText("")
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"Table:", None))
        self.readButton.setText(QCoreApplication.translate("MainWindow", u"Read", None))
        self.writeButton.setText(QCoreApplication.translate("MainWindow", u"Write", None))
        self.readWriteButton.setText(QCoreApplication.translate("MainWindow", u"Read-Write", None))
        self.menuDevice.setTitle(QCoreApplication.translate("MainWindow", u"&Device", None))
        self.menuToo_ls.setTitle(QCoreApplication.translate("MainWindow", u"Too&ls", None))
    # retranslateUi




# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import os
import re
import subprocess
import sys
import tempfile
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

VERSION = 6


DESC = """Runs uic on a set of UI files and displays the resulting widgets."""


TEMP_DIR = Path(tempfile.gettempdir())


def get_class_name(file: Path) -> tuple[str | None, str | None]:
    """Return class name and widget name of UI file."""
    pattern = re.compile(r'^\s*<widget class="(\w+)" name="(\w+)"\s*>.*$')
    for line in Path(file).read_text().splitlines():
        match = pattern.match(line)
        if match:
            return (match.group(1), match.group(2))
    return (None, None)


def test_file(file: str, uic: bool = False) -> bool:
    """Run uic on a UI file and show the resulting UI."""
    path = Path(file)
    (klass, name) = get_class_name(path)
    if not klass:
        print(f'{file} does not appear to be a UI file', file=sys.stderr)
        return False
    py_klass = f'Ui_{name}'
    py_file_basename = py_klass.lower()
    py_file = TEMP_DIR / (py_file_basename + '.py')
    py_main = TEMP_DIR / 'main.py'
    cmd = ['uic', '-g', 'python'] if uic else [f'pyside{VERSION}-uic']
    cmd.extend(['-o', os.fspath(py_file), file])
    try:
        subprocess.call(cmd)
    except FileNotFoundError as e:
        print(str(e) + " (try -u for uic)", file=sys.stderr)
        return False
    main_source = dedent(f'''\
        import sys
        from PySide{VERSION}.QtWidgets import QApplication, {klass}
        from {py_file_basename} import {py_klass}

        if __name__ == "__main__":
            app = QApplication(sys.argv)
            ui = {py_klass}()
            widget = {klass}()
            ui.setupUi(widget)
            widget.show()
            sys.exit(app.exec())''')
    py_main.write_text(main_source)
    exit_code = subprocess.call([sys.executable, os.fspath(py_main)])
    py_main.unlink()
    py_file.unlink()
    return exit_code == 0


if __name__ == '__main__':
    argument_parser = ArgumentParser(description=DESC,
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument('--uic', '-u', action='store_true',
                                 help='Use uic instead of pyside-uic')
    argument_parser.add_argument("files", help="UI Files",
                                 nargs='+', type=str)
    options = argument_parser.parse_args()
    failed = 0
    count = len(options.files)
    for i, file in enumerate(options.files):
        print(f'{i + 1}/{count} {file}')
        if not test_file(file, options.uic):
            failed += 1
    if failed != 0:
        print(f'{failed}/{count} failed.')
    sys.exit(failed)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for PySide API2 support'''


import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QIntValidator, QValidator
from PySide6.QtWidgets import QWidget, QSpinBox, QApplication

from helper.usesqapplication import UsesQApplication


class WidgetValidatorQInt(QWidget, QIntValidator):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        QIntValidator.__init__(self, parent)


class WidgetValidatorQSpinBox(QSpinBox):
    def __init__(self, parent=None):
        QSpinBox.__init__(self, parent)

    def fixup(self, text):
        print("It was called!")


class DoubleQObjectInheritanceTest(UsesQApplication):

    def testDouble(self):
        '''Double inheritance from QObject classes'''

        obj = WidgetValidatorQInt()

        # QIntValidator methods
        state, string, number = obj.validate('Test', 0)
        self.assertEqual(state, QValidator.State.Invalid)
        state, string, number = obj.validate('33', 0)
        self.assertEqual(state, QValidator.State.Acceptable)

    def testQSpinBox(self):
        obj = WidgetValidatorQSpinBox()

        obj.setRange(1, 10)
        obj.setValue(0)
        self.assertEqual(obj.value(), 1)


class QClipboardTest(UsesQApplication):

    def testQClipboard(self):
        # skip this test on macOS because the clipboard is not available during the ssh session
        # this cause problems in the buildbot
        if sys.platform == 'darwin':
            return
        clip = QApplication.clipboard()
        clip.setText("Testing this thing!")

        text, subtype = clip.text("")
        self.assertEqual(subtype, "plain")
        self.assertEqual(text, "Testing this thing!")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import SizeF


class NumericalTypedefTest(unittest.TestCase):

    def testNumericalTypedefExact(self):
        width, height = (1.1, 2.2)
        size = SizeF(width, height)
        self.assertEqual(size.width(), width)
        self.assertEqual(size.height(), height)

    def testNumericalTypedefConvertible(self):
        width, height = (1, 2)
        size = SizeF(width, height)
        self.assertEqual(size.width(), float(width))
        self.assertEqual(size.height(), float(height))

    def testNumericalTypedefOfUnsignedShort(self):
        self.assertEqual(SizeF.passTypedefOfUnsignedShort(123), 123)
        self.assertEqual(SizeF.passTypedefOfUnsignedShort(321), 321)
        self.assertNotEqual(SizeF.passTypedefOfUnsignedShort(123), 0)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtGui import QTextBlockUserData, QTextCursor
from PySide6.QtWidgets import QTextEdit


class TestUserData(QTextBlockUserData):
    def __init__(self, data):
        super().__init__()
        self.data = data


class TestUserDataRefCount(UsesQApplication):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefcount(self):
        textedit = QTextEdit()
        textedit.setReadOnly(True)
        doc = textedit.document()
        cursor = QTextCursor(doc)
        cursor.insertText("PySide Rocks")
        ud = TestUserData({"Life": 42})
        base_ref_count = sys.getrefcount(ud)
        cursor.block().setUserData(ud)
        self.assertEqual(sys.getrefcount(ud), base_ref_count + 1)
        ud2 = cursor.block().userData()
        self.assertEqual(sys.getrefcount(ud), base_ref_count + 2)
        self.udata = weakref.ref(ud, None)
        del ud, ud2
        delta = 1 if sys.version_info >= (3, 14) else 0
        self.assertEqual(sys.getrefcount(self.udata()), base_ref_count + delta)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for the QIOPipe class'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)


from PySide6.QtCore import QIODevice, QIOPipe


class QIOPipeTest(unittest.TestCase):
    def setUp(self) -> None:
        self.pipe = QIOPipe()
        self.pipe.open(QIODevice.OpenModeFlag.ReadWrite)
        return super().setUp()

    def tearDown(self) -> None:
        super().tearDown()

    def ready_read_bytes_written(self):
        received_data = self.pipe.end2().readAll().data()
        self.assertEqual(received_data, self.data)

    def test_readyRead(self):
        self.data = b"Hello, World!"
        self.pipe.end2().readyRead.connect(self.ready_read_bytes_written)
        self.pipe.end1().write(self.data)

    def test_bytesWritten(self):
        self.data = b"Hello, World!"
        self.pipe.end2().bytesWritten.connect(self.ready_read_bytes_written)
        self.pipe.end1().write(self.data)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QObject  # noqa: E402


class MyObject(QObject):
    def __init__(self, other=None):
        super().__init__(None)
        self._o = other


class TestDestructor(unittest.TestCase):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testReference(self):
        o = QObject()
        base_ref_count = sys.getrefcount(o)
        m = MyObject(o)
        self.assertEqual(sys.getrefcount(o), base_ref_count + 1)
        del m
        self.assertEqual(sys.getrefcount(o), base_ref_count)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import logging
import os

from build_scripts.log import log
from build_scripts.utils import expand_clang_variables
from coin.instructions_utils import (CI, call_setup, get_architecture,
                                     get_python_version, remove_variables)

if __name__ == "__main__":
    ci = CI()
    log.setLevel(logging.INFO)
    # Remove some environment variables that impact cmake
    arch = get_architecture(ci)
    expand_clang_variables(arch)

    remove_variables(["CC", "CXX"])

    python_ver = get_python_version(ci)

    os.chdir(ci.ENV_AGENT_DIR)
    testRun = 0

    call_setup(python_ver, ci, "TEST", log)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import ctypes
import numpy
from OpenGL.GL import (GL_COLOR_BUFFER_BIT, GL_CULL_FACE, GL_CW,
                       GL_DEPTH_BUFFER_BIT, GL_DEPTH_TEST, GL_FALSE, GL_FLOAT,
                       GL_TEXTURE_2D, GL_TRIANGLES)

from PySide6.QtGui import QMatrix4x4, QOpenGLContext
from PySide6.QtOpenGL import (QOpenGLBuffer, QOpenGLShader,
                              QOpenGLShaderProgram, QOpenGLVertexArrayObject)
from shiboken6 import VoidPtr


VERTEXSHADER_SOURCE = """attribute highp vec4 vertex;
attribute lowp vec2 coord;
varying lowp vec2 v_coord;
uniform highp mat4 matrix;
void main() {
   v_coord = coord;
   gl_Position = matrix * vertex;
}
"""


FRAGMENTSHADER_SOURCE = """varying lowp vec2 v_coord;
uniform sampler2D sampler;
void main() {
   gl_FragColor = vec4(texture2D(sampler, v_coord).rgb, 1.0);
}
"""


FLOAT_SIZE = ctypes.sizeof(ctypes.c_float)


VERTEXES = numpy.array([-0.5, 0.5, 0.5, 0.5, -0.5, 0.5, -0.5, -0.5, 0.5,
                        0.5, -0.5, 0.5, -0.5, 0.5, 0.5, 0.5, 0.5, 0.5,
                        -0.5, -0.5, -0.5, 0.5, -0.5, -0.5, -0.5, 0.5, -0.5,
                        0.5, 0.5, -0.5, -0.5, 0.5, -0.5, 0.5, -0.5, -0.5,

                        0.5, -0.5, -0.5, 0.5, -0.5, 0.5, 0.5, 0.5, -0.5,
                        0.5, 0.5, 0.5, 0.5, 0.5, -0.5, 0.5, -0.5, 0.5,
                        -0.5, 0.5, -0.5, -0.5, -0.5, 0.5, -0.5, -0.5, -0.5,
                        -0.5, -0.5, 0.5, -0.5, 0.5, -0.5, -0.5, 0.5, 0.5,

                        0.5, 0.5, -0.5, -0.5, 0.5, 0.5, -0.5, 0.5, -0.5,
                        -0.5, 0.5, 0.5, 0.5, 0.5, -0.5, 0.5, 0.5, 0.5,
                        -0.5, -0.5, -0.5, -0.5, -0.5, 0.5, 0.5, -0.5, -0.5,
                        0.5, -0.5, 0.5, 0.5, -0.5, -0.5, -0.5, -0.5, 0.5],
                       dtype=numpy.float32)


TEX_COORDS = numpy.array([0.0, 0.0, 1.0, 1.0, 1.0, 0.0,
                          1.0, 1.0, 0.0, 0.0, 0.0, 1.0,
                          1.0, 1.0, 1.0, 0.0, 0.0, 1.0,
                          0.0, 0.0, 0.0, 1.0, 1.0, 0.0,

                          1.0, 1.0, 1.0, 0.0, 0.0, 1.0,
                          0.0, 0.0, 0.0, 1.0, 1.0, 0.0,
                          0.0, 0.0, 1.0, 1.0, 1.0, 0.0,
                          1.0, 1.0, 0.0, 0.0, 0.0, 1.0,

                          0.0, 1.0, 1.0, 0.0, 1.0, 1.0,
                          1.0, 0.0, 0.0, 1.0, 0.0, 0.0,
                          1.0, 0.0, 1.0, 1.0, 0.0, 0.0,
                          0.0, 1.0, 0.0, 0.0, 1.0, 1.0], dtype=numpy.float32)


class CubeRenderer():
    def __init__(self, offscreenSurface):
        self.m_angle = 0
        self.m_offscreenSurface = offscreenSurface
        self.m_context = None
        self.m_program = None
        self.m_vbo = None
        self.m_vao = None
        self.m_matrixLoc = 0
        self.m_proj = QMatrix4x4()

    def __del__(self):
        # Use a temporary offscreen surface to do the cleanup. There may not
        # be a native window surface available anymore at self stage.
        self.m_context.makeCurrent(self.m_offscreenSurface)
        del self.m_program
        del self.m_vbo
        del self.m_vao
        self.m_context.doneCurrent()

    def init(self, w, share):
        self.m_context = QOpenGLContext()
        self.m_context.setShareContext(share)
        self.m_context.setFormat(w.requestedFormat())
        self.m_context.create()
        if not self.m_context.makeCurrent(w):
            return

        f = self.m_context.functions()
        f.glClearColor(0.0, 0.1, 0.25, 1.0)
        f.glViewport(0, 0, w.width() * w.devicePixelRatio(),
                     w.height() * w.devicePixelRatio())

        self.m_program = QOpenGLShaderProgram()
        self.m_program.addCacheableShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Vertex,
                                                        VERTEXSHADER_SOURCE)
        self.m_program.addCacheableShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Fragment,
                                                        FRAGMENTSHADER_SOURCE)
        self.m_program.bindAttributeLocation("vertex", 0)
        self.m_program.bindAttributeLocation("coord", 1)
        self.m_program.link()
        self.m_matrixLoc = self.m_program.uniformLocation("matrix")

        self.m_vao = QOpenGLVertexArrayObject()
        self.m_vao.create()

        self.m_vbo = QOpenGLBuffer()
        self.m_vbo.create()
        self.m_vbo.bind()

        vertexCount = 36
        self.m_vbo.allocate(FLOAT_SIZE * vertexCount * 5)
        vertex_data = VERTEXES.tobytes()
        tex_coord_data = TEX_COORDS.tobytes()
        self.m_vbo.write(0, VoidPtr(vertex_data),
                         FLOAT_SIZE * vertexCount * 3)
        self.m_vbo.write(FLOAT_SIZE * vertexCount * 3,
                         VoidPtr(tex_coord_data),
                         FLOAT_SIZE * vertexCount * 2)
        self.m_vbo.release()

        if self.m_vao.isCreated():
            self.setupVertexAttribs()

    def resize(self, w, h):
        self.m_proj.setToIdentity()
        self.m_proj.perspective(45, w / float(h), 0.01, 100.0)

    def setupVertexAttribs(self):
        self.m_vbo.bind()
        self.m_program.enableAttributeArray(0)
        self.m_program.enableAttributeArray(1)
        f = self.m_context.functions()

        null = VoidPtr(0)
        pointer = VoidPtr(36 * 3 * FLOAT_SIZE)
        f.glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, null)
        f.glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, pointer)
        self.m_vbo.release()

    def render(self, w, share, texture):
        if not self.m_context:
            self.init(w, share)

        if not self.m_context.makeCurrent(w):
            return

        f = self.m_context.functions()
        f.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        if texture:
            f.glBindTexture(GL_TEXTURE_2D, texture)
            f.glFrontFace(GL_CW)  # because our cube's vertex data is such
            f.glEnable(GL_CULL_FACE)
            f.glEnable(GL_DEPTH_TEST)

            self.m_program.bind()
            # If VAOs are not supported, set the vertex attributes every time.
            if not self.m_vao.isCreated():
                self.setupVertexAttribs()

            m = QMatrix4x4()
            m.translate(0, 0, -2)
            m.rotate(90, 0, 0, 1)
            m.rotate(self.m_angle, 0.5, 1, 0)
            self.m_angle += 0.5

            self.m_program.setUniformValue(self.m_matrixLoc, self.m_proj * m)

            # Draw the cube.
            f.glDrawArrays(GL_TRIANGLES, 0, 36)

        self.m_context.swapBuffers(w)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for QHostInfo.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtCore import (QCoreApplication, QElapsedTimer, QObject, QThread,
                            Slot, SLOT)
from PySide6.QtNetwork import QHostInfo


HOST = 'www.qt.io'


TIMEOUT = 30000


class Receiver(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._slot_called = False

    def slot_called(self):
        return self._slot_called

    @Slot(QHostInfo)
    def info_received(self, host_info):
        name = host_info.hostName()
        if host_info.error() == QHostInfo.HostInfoError.NoError:
            addresses = [a.toString() for a in host_info.addresses()]
            addresses_str = ', '.join(addresses)
            print(f'"{name}" resolved to {addresses_str}')
        else:
            error = host_info.errorString()
            print(f'Unable to resolve "{name}": {error}', file=sys.stderr)
        self._slot_called = True


class QHostInfoTest(UsesQApplication):
    '''Test case for QHostInfo.'''
    def setUp(self):
        UsesQApplication.setUp(self)
        self._timer = QElapsedTimer()

    def testStringBasedLookup(self):
        receiver = Receiver()
        self._timer.restart()
        QHostInfo.lookupHost(HOST, receiver, SLOT('info_received(QHostInfo)'))
        while not receiver.slot_called() and self._timer.elapsed() < TIMEOUT:
            QCoreApplication.processEvents()
            QThread.msleep(10)
        print(f'String-based: Elapsed {self._timer.elapsed()}ms')
        self.assertTrue(receiver.slot_called())

    def testCallableLookup(self):
        receiver = Receiver()
        self._timer.restart()
        QHostInfo.lookupHost(HOST, receiver.info_received)
        while not receiver.slot_called() and self._timer.elapsed() < TIMEOUT:
            QCoreApplication.processEvents()
            QThread.msleep(10)
        print(f'Callable: Elapsed {self._timer.elapsed()}ms')
        self.assertTrue(receiver.slot_called())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Connecting lambda to signals'''

import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QObject, Signal, SIGNAL, QProcess

from helper.usesqapplication import UsesQApplication


class Sender(QObject):
    void_signal = Signal()
    int_signal = Signal(int)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._delayed_int = 0

    def emit_void(self):
        self.void_signal.emit()

    def emit_int(self, v):
        self.int_signal.emit(v)


class Receiver(QObject):

    def __init__(self, *args):
        super().__init__(*args)


class BasicCase(unittest.TestCase):

    def testSimplePythonSignalNoArgs(self):
        # Connecting a lambda to a simple python signal without arguments
        receiver = Receiver()
        sender = Sender()
        sender.void_signal.connect(lambda: setattr(receiver, 'called', True))
        sender.emit_void()
        self.assertTrue(receiver.called)

    def testSimplePythonSignal(self):
        # Connecting a lambda to a simple python signal witharguments
        receiver = Receiver()
        sender = Sender()
        arg = 42
        sender.int_signal.connect(lambda x: setattr(receiver, 'arg', arg))
        sender.emit_int(arg)
        self.assertEqual(receiver.arg, arg)

    def testSimplePythonSignalNoArgsString(self):
        # Connecting a lambda to a simple python signal without arguments
        receiver = Receiver()
        sender = Sender()
        QObject.connect(sender, SIGNAL('void_signal()'),
                        lambda: setattr(receiver, 'called', True))
        sender.emit_void()
        self.assertTrue(receiver.called)

    def testSimplePythonSignalString(self):
        # Connecting a lambda to a simple python signal witharguments
        receiver = Receiver()
        sender = Sender()
        arg = 42
        QObject.connect(sender, SIGNAL('int_signal(int)'),
                        lambda x: setattr(receiver, 'arg', arg))
        sender.emit_int(arg)
        self.assertEqual(receiver.arg, arg)


class QtSigLambda(UsesQApplication):

    qapplication = True

    def testWithArgs(self):
        '''Connecting a lambda to a signal with and without arguments'''
        proc = QProcess()
        dummy = Receiver()
        proc.started.connect(lambda: setattr(dummy, 'called', True))
        proc.finished.connect(lambda x: setattr(dummy, 'exit_code', x))

        proc.start(sys.executable, ['-c', '""'])
        self.assertTrue(proc.waitForStarted())
        self.assertTrue(proc.waitForFinished())

        self.assertTrue(dummy.called)
        self.assertEqual(dummy.exit_code, proc.exitCode())

    def testRelease(self):
        """PYSIDE-2646: Test whether main thread target slot lambda/methods
           (and their captured objects) are released by the signal manager
           after a while."""

        def do_connect(sender):
            receiver = Receiver()
            sender.void_signal.connect(lambda: setattr(receiver, 'called', True))
            return receiver

        sender = Sender()
        receiver = weakref.ref(do_connect(sender))
        sender.emit_void()
        self.assertTrue(receiver().called)
        del sender
        for i in range(3):
            if not receiver():
                break
            QCoreApplication.processEvents()
        self.assertFalse(receiver())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Color, Brush


class TestNonZeroOperator(unittest.TestCase):
    def testColor(self):
        """Color has a Qt-style isNull()"""
        c = Color()
        self.assertFalse(c)
        c = Color(2)
        self.assertTrue(c)

    def testBrush(self):
        """Brush enables its operator bool in the typesystem XML"""
        b = Brush()
        self.assertFalse(b)
        b = Brush(Color(2))
        self.assertTrue(b)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

# broken_links.py
# This script is used to find the broken links in the generated sphinx documentation.

# Pre-requisites before running this script:
#
# 1. Generate the HTML documentation as given in
#    https://doc.qt.io/qtforpython-6/gettingstarted/index.html#building-the-documentation
#    . This will generate a folder inside 'pyside-setup' called 'html'.
# 2. Run sphinx-build with linkcheck builder to generate an output.json file inside 'html' folder.
#    The command to run is:
#    sphinx-build -b linkcheck -j auto -n -c ./html/pyside6/base <folder_of_rst> ./html
#
# The final output will be <output_folder>/broken_links.json

import json
import argparse
from pathlib import Path


def find_broken_links(file_path):
    with open(file_path, 'r') as file:
        lines = file.readlines()

    broken_links = []
    for line in lines:
        entry = json.loads(line)
        if entry.get("status") == "broken":
            broken_links.append(entry)

    return broken_links


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Find broken links in a JSON file.")
    parser.add_argument('--input-file', type=str, required=True, help='Path to the input JSON file')
    parser.add_argument('--output-folder', type=str, default=Path(__file__).resolve().parents[2],
                        help='Path to the output folder')
    args = parser.parse_args()

    broken_links = find_broken_links(args.input_file)

    output_file_path = Path(args.output_folder) / 'broken_links.json'

    if broken_links:
        with open(output_file_path, 'w') as outfile:
            json.dump(broken_links, outfile, indent=4)
        print(f"Broken links written to {output_file_path}")
    else:
        print("No broken links found.")



#
# This code is a modified version of the code found in sphinx distribution
# the same license of sphinx distribution applies to this file as well.
#

import os
import sys
import codecs
from os import path

from docutils import nodes
from docutils.parsers.rst import Directive, directives

from sphinx.util import parselinenos


class PySideInclude(Directive):
    """
    Like ``.. include:: :literal:``, but only warns if the include file is
    not found, and does not raise errors.  Also has several options for
    selecting what to include.
    """

    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {
        'linenos': directives.flag,
        'tab-width': int,
        'language': directives.unchanged_required,
        'encoding': directives.encoding,
        'pyobject': directives.unchanged_required,
        'lines': directives.unchanged_required,
        'start-after': directives.unchanged_required,
        'end-before': directives.unchanged_required,
        'prepend': directives.unchanged_required,
        'append': directives.unchanged_required,
        'snippet': directives.unchanged_required,
    }

    def run(self):
        document = self.state.document
        filename = self.arguments[0]
        if not document.settings.file_insertion_enabled:
            return [document.reporter.warning('File insertion disabled',
                                              line=self.lineno)]
        env = document.settings.env
        if filename.startswith('/') or filename.startswith(os.sep):
            rel_fn = filename[1:]
        else:
            docdir = path.dirname(env.doc2path(env.docname, base=None))
            rel_fn = path.join(docdir, filename)
        try:
            fn = path.join(env.srcdir, rel_fn)
        except UnicodeDecodeError:
            # the source directory is a bytestring with non-ASCII characters;
            # let's try to encode the rel_fn in the file system encoding
            rel_fn = rel_fn.encode(sys.getfilesystemencoding())
            fn = path.join(env.srcdir, rel_fn)

        if 'pyobject' in self.options and 'lines' in self.options:
            return [document.reporter.warning(
                'Cannot use both "pyobject" and "lines" options',
                line=self.lineno)]

        encoding = self.options.get('encoding', env.config.source_encoding)
        codec_info = codecs.lookup(encoding)
        try:
            f = codecs.StreamReaderWriter(open(fn, 'Ub'),
                                          codec_info[2], codec_info[3], 'strict')
            lines = f.readlines()
            f.close()
        except (IOError, OSError):
            return [document.reporter.warning(
                'Include file %r not found or reading it failed' % filename,
                line=self.lineno)]
        except UnicodeError:
            return [document.reporter.warning(
                'Encoding %r used for reading included file %r seems to '
                'be wrong, try giving an :encoding: option' %
                (encoding, filename))]

        objectname = self.options.get('pyobject')
        if objectname is not None:
            from sphinx.pycode import ModuleAnalyzer
            analyzer = ModuleAnalyzer.for_file(fn, '')
            tags = analyzer.find_tags()
            if objectname not in tags:
                return [document.reporter.warning(
                    'Object named %r not found in include file %r' %
                    (objectname, filename), line=self.lineno)]
            else:
                lines = lines[tags[objectname][1] - 1: tags[objectname][2] - 1]

        linespec = self.options.get('lines')
        if linespec is not None:
            try:
                linelist = parselinenos(linespec, len(lines))
            except ValueError as err:
                return [document.reporter.warning(str(err), line=self.lineno)]
            lines = [lines[i] for i in linelist]

        startafter = self.options.get('start-after')
        endbefore = self.options.get('end-before')
        prepend = self.options.get('prepend')
        append = self.options.get('append')
        snippet = self.options.get('snippet')

        if snippet:
            startafter = "//![%s]" % snippet
            endbefore = "//![%s]" % snippet

        if startafter is not None or endbefore is not None:
            use = not startafter
            res = []
            for line in lines:
                if not use and startafter and startafter in line:
                    use = True
                elif use and endbefore and endbefore in line:
                    use = False
                    break
                elif use:
                    if not line.startswith("//!"):
                        res.append(line)
            lines = res

        if prepend:
            lines.insert(0, prepend + '\n')
        if append:
            lines.append(append + '\n')

        text = ''.join(lines)
        if self.options.get('tab-width'):
            text = text.expandtabs(self.options['tab-width'])
        retnode = nodes.literal_block(text, text, source=fn)
        retnode.line = 1
        retnode.attributes['line_number'] = self.lineno
        if self.options.get('language', ''):
            retnode['language'] = self.options['language']
        if 'linenos' in self.options:
            retnode['linenos'] = True
        document.settings.env.note_dependency(rel_fn)
        return [retnode]


def setup(app):
    app.add_directive('pysideinclude', PySideInclude)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QStandardItemModel, QStandardItem


class MyItemModel(QStandardItemModel):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.appendRow([QStandardItem('Item 1'),])

    def mimeTypes(self):
        mtypes = super(MyItemModel, self).mimeTypes()
        mtypes.append('application/my-form')
        return mtypes

    def mimeData(self, indexes):
        self.__mimedata = super(MyItemModel, self).mimeData(indexes)
        self.__mimedata.setData('application/my-form', bytes('hi', "UTF-8"))
        return self.__mimedata


class TestBug660(unittest.TestCase):
    '''QMimeData type deleted prematurely when overriding mime-type in QStandardItemModel
       drag and drop'''
    def testIt(self):
        model = MyItemModel()
        model.mimeData([model.index(0, 0)])  # if it doesn't raise an exception it's all right!


if __name__ == '__main__':
    unittest.main()
